package cc.smtweb.framework.core.db.cache;

import cc.smtweb.framework.core.annotation.SwCache;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.ResultSetExtractor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/1/12 18:34
 */
@SwCache(ident = "ASP_MODEL_TABLE", title = "数据库表定义")
public class ModelTableCache extends AbstractCache<ModelTable> {
    private final static String mk = "k";
    private final static String md = "d";
    private final static String mc = "c";

    public static ModelTableCache getInstance() {
        return CacheManager.getIntance().getCache(ModelTableCache.class);
    }

    public ModelTableCache() {
        regMap(mk, k-> k.getName().toUpperCase());
        regList(md, k-> String.valueOf(k.getDbId()));
        regList(mc, k-> String.valueOf(k.getMcId()));
//        regList(mf, k-> k.get);
    }

    @Override
    protected String getId(ModelTable bean) {
        return String.valueOf(bean.getId());
    }

    @Override
    protected List<ModelTable> loadAll() {
        return DbEngine.getInstance().query("SELECT\n" +
                "t.tb_id,\n" +
                "t.tb_prj_id,\n" +
                "t.tb_mc_id,\n" +
                "t.tb_db_id,\n" +
                "t.tb_extends,\n" +
                "t.tb_name,\n" +
                "t.tb_title,\n" +
                "t.tb_abbr,\n" +
                "t.tb_type,\n" +
                "t.tb_need_cache,\n" +
                "t.tb_content,\n" +
                "t.tb_create_uid,\n" +
                "t.tb_update_uid,\n" +
                "t.tb_create_at,\n" +
                "t.tb_update_at\n" +
                "from asp_model_table t\n", new ResultSetExtractor<List<ModelTable>>() {
            @Override
            public List<ModelTable> extractData(ResultSet rs) throws SQLException, DataAccessException {
                List<ModelTable> list = new ArrayList<>();
                while (rs.next()) {
                    ModelTable table = new ModelTable();
                    list.add(table);
                    table.setId(rs.getLong("tb_id"));
                    table.setDbId(rs.getLong("tb_db_id"));
                    table.setPrjId(rs.getLong("tb_prj_id"));
                    table.setMcId(rs.getLong("tb_mc_id"));
                    table.setExtends(rs.getString("tb_extends"));
                    table.setName(rs.getString("tb_name").toUpperCase());
                    table.setTitle(rs.getString("tb_title"));
                    table.setAbbr(rs.getString("tb_abbr"));
                    table.setType(rs.getInt("tb_type"));
                    table.setNeedCache(rs.getInt("tb_need_cache") == 1);
                    table.setCreateUid(rs.getLong("tb_create_uid"));
                    table.setUpdateUid(rs.getLong("tb_update_uid"));
                    table.setCreateAt(rs.getLong("tb_create_at"));
                    table.setUpdateAt(rs.getLong("tb_update_at"));
                    table.setContent(rs.getString("tb_content"));
                }
                return list;
            }
        });
    }

    public final ModelTable getByName(String key) {
        return getByKey(mk, key.toUpperCase());
    }
    public final Set<ModelTable> getDbTables(long dbId) {
        return getListByKey(md, String.valueOf(dbId));
    }

    public final Set<ModelTable> getTablesByMc(long mcId) {
        return getListByKey(mc, String.valueOf(mcId));
    }

    public final List<ModelTable> getTablesByMc(long mcId, Comparator<ModelTable> comparator) {
        Set<ModelTable> set = getListByKey(mc, String.valueOf(mcId));
        if (set == null || set.isEmpty()) return null;
        List<ModelTable> list = new ArrayList<>(set);
        list.sort(comparator);
        return list;
    }
}
