package cc.smtweb.framework.core.db.config;

import cc.smtweb.framework.core.common.SwEnum;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
//通过@PropertySource注解指定要读取的yaml配置文件，默认读取src\main\resources\application.yml配置
@PropertySource(value = "classpath:config/design_db.yaml", factory = YamlPropertyLoaderFactory.class)
@ConfigurationProperties(prefix = "design")
public class DesignConfig {
    private List<SwEnum.DataType> dataTypes;

    private Map<String, SwEnum.DataType> mapType = null;

    private void adjustMap() {
        if (mapType == null) {
            synchronized (DesignConfig.class) {
                if (mapType == null) {
                    Map<String, SwEnum.DataType> map = new HashMap<>();
                    for (SwEnum.DataType type : dataTypes) {
                        map.put(type.getType(), type);
                    }
                    mapType = map;
                }
            }
        }
    }

    public SwEnum.DataType getDataType(String type) {
        adjustMap();
        return mapType.get(type);
    }

    public List<SwEnum.DataType> getDataTypes() {
        return dataTypes;
    }
}
