package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwException;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.session.UserSession;

/**
 * Created by Akmm at 2022/3/2 10:39
 * 通用业务mvc总调度
 */
public abstract class AbstractCompService {
    public final static String TYPE_LIST = "list";
    public final static String TYPE_COMBO = "combo";
    public final static String TYPE_TREE = "tree";
    public final static String TYPE_LOAD = "load";
    public final static String TYPE_SAVE = "save";
    public final static String TYPE_DEL = "del";

    protected abstract IHandler createHandler(String type);

    private IHandler getHandler(SwMap params, UserSession us, String type) throws Exception {
        IHandler handler = createHandler(type);
        if (handler == null) throw new SwException("暂不支持此类服务：" + type);
        if (params == null) params = new SwMap();
        if (us == null) us = UserSession.createSys();
        handler.init(params, us);
        return handler;
    }

    private R pageHandler(SwMap params, UserSession us, String type) {
        try {
            IHandler handler = getHandler(params, us, type);
            return handler.doWork();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }

    //保存
    public R save(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_SAVE);
    }

    //树，换爹
    public R trcp(@SwBody SwMap params, UserSession us) {
        try {
            DefaultSaveHandler handler = (DefaultSaveHandler) getHandler(params, us, TYPE_SAVE);
            return handler.changeParent();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }

    //读取
    public R load(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LOAD);
    }

    //删除
    public R del(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_DEL);
    }

    //列表数据
    public R list(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LIST);
    }

    //列表总记录数及合计栏
    public R listTotal(@SwBody SwMap params, UserSession us) {
        try {
            AbstractListHandler handler = (AbstractListHandler) getHandler(params, us, TYPE_LIST);
            return handler.getTotal();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }

    //combo数据
    public R combo(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_COMBO);
    }

    //combo总记录数及合计栏
    public R comboTotal(@SwBody SwMap params, UserSession us) {
        try {
            AbstractListHandler handler = (AbstractListHandler) getHandler(params, us, TYPE_COMBO);
            return handler.getTotal();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }

    //combo数据过滤
    public R comboFilter(@SwBody SwMap params, UserSession us) {
        try {
            DefaultComboHandler handler = (DefaultComboHandler) getHandler(params, us, TYPE_COMBO);
            return handler.filter();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }

    //树数据
    public R tree(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_TREE);
    }

    //树过滤
    public R treeFilter(@SwBody SwMap params, UserSession us) {
        try {
            AbstractTreeHandler handler = (AbstractTreeHandler) getHandler(params, us, TYPE_TREE);
            return handler.filter();
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }
}
