package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.cache.SessionCache;
import cc.smtweb.framework.core.cache.SessionCacheFactory;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.mvc.service.list.FooterField;
import cc.smtweb.framework.core.session.UserSession;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Akmm at 2022/3/2 19:44
 * 列表服务
 */
@Slf4j
public abstract class AbstractTreeHandler<T> extends AbstractHandler {

    @Override
    public R doWork() throws Exception {
        return R.success(data());
    }

    //树过滤
    public R filter() throws Exception {
        List<T> rows = filterData();
        List<SwMap> listRet = buildNodes(rows, true);
        return R.success(listRet);
    }

    protected List<SwMap> data() throws Exception {
        List<T> rows = getChildren(params.readLong("parent_id"));
        List<SwMap> listRet = buildNodes(rows, params.readBool("lazy"));

        return listRet;
    }

    //搜索
    protected abstract List<T> filterData();
    //获取指定节点的下级节点
    protected abstract List<T> getChildren(long id);
    protected List<T> getChildren(T bean) {
        return getChildren(getId(bean));
    }

    //根据bean，构建treenode
    private List<SwMap> buildNodes(List<T> rows, boolean lazy) {
        List<SwMap> listRet = new ArrayList<>();
        if (rows == null || rows.isEmpty()) {
            return listRet;
        }
        for (T row : rows) {
            SwMap node = new SwMap();
            node.put("id", getId(row));
            node.put("text", getText(row));
            List<T> children = getChildren(row);
            node.put("leaf", children == null || children.isEmpty());
            node.put("bean", row);
            buildNode(node, row);
            listRet.add(node);
            if (!lazy) {
                List<SwMap> list = buildNodes(children, lazy);
                node.put("children", list);
            }
        }
        return listRet;
    }

    //根据bean，构建treenode
    protected void buildNode(SwMap node, T bean) {}

    protected abstract long getId(T bean);

    protected abstract String getText(T bean);
}
