package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelLinkName;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * Created by Akmm at 2022/3/2 19:52
 * 默认实体实现
 */
public class DefaultListHandler<T extends DefaultEntity> extends AbstractListHandler {
    protected String tableName;

    public DefaultListHandler(String tableName) {
        this.tableName = tableName;
    }

    @Override
    protected SqlPara buildSqlPara() throws Exception {
        EntityDao dao = DbEngine.getInstance().findDao(tableName);
        StringBuilder sql = new StringBuilder(512);
        List<Object> args = new ArrayList<>();
        dao.handleSelect(sql, null);

        StringBuilder sqlCondition = new StringBuilder(128);
        buildCondition(sqlCondition, args);
        String filter = sqlCondition.toString().trim();
        if (StringUtils.isNotEmpty(filter)) {
            if (filter.startsWith("or ")) {
                sql.append("\nwhere ").append(filter.substring(3));
            } else if (filter.startsWith("and ")) {
                sql.append("\nwhere ").append(filter.substring(4));
            } else if (!filter.startsWith("where ")) {
                sql.append("\nwhere ").append(filter);
            } else {
                sql.append(filter);
            }
        }

        buildOrder(sql);

        return new SqlPara(sql.toString(), args.toArray());
    }

    protected void buildCondition(StringBuilder sql, List<Object> args) {
    }

    protected void buildOrder(StringBuilder sql) {
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table == null) return;
        sql.append(" order by ");

        ModelField field = table.findFieldByType(SwEnum.FieldType.CODE.value);
        if (field != null) sql.append(field.getName()).append(",");
        sql.append(table.getIdField());
    }

    @Override
    protected void afterQuery(List<SwMap> listData) {
        super.afterQuery(listData);
        //添加关联字段值
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table == null) return;
        List<ModelLinkName> listLink = table.findLinkeNames();
        if (listLink.isEmpty()) return;

        //有缓存的，从缓存拿，无缓存的放Map，最后从数据库去拿
        Map<String, List<String>> mapIds = new HashMap<>();
        for (SwMap row : listData) {
            for (ModelLinkName l : listLink) {
                String value = row.readString(l.getFieldName());
                if (StringUtils.isEmpty(value)) continue;
                String[] ids = StringUtils.split(value, ",");
                if (l.getLinkTable().isNeedCache()) {
                    EntityDao dao = DbEngine.getInstance().findDao(l.getLinkTable().getName());
                    AbstractCache cache = CacheManager.getIntance().getCache(l.getLinkTable().getName());
                    String names = "";
                    for (String sId : ids) {
                        Object b = cache.get(sId);
                        String sn = (String) dao.readValue(b, l.getLinkNameField());
                        if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                    }
                    if (StringUtils.isNotEmpty(names)) {
                        row.put(l.getFieldName() + "_text", names.substring(1));
                    } else {
                        row.put(l.getFieldName() + "_text", "");
                    }
                } else {
                    List<String> list = mapIds.computeIfAbsent(l.getLinkTable().getName(), k -> new ArrayList<>());
                    Collections.addAll(list, ids);
                }
            }
        }

        if (mapIds.isEmpty()) return;
        //数据库查询
        Map<String, Map<String, String>> mapValue = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : mapIds.entrySet()) {
            EntityDao dao = DbEngine.getInstance().findDao(entry.getKey());
            mapValue.put(entry.getKey(), dao.queryNames(entry.getValue()));
        }
        //加值
        for (SwMap row : listData) {
            for (ModelLinkName l : listLink) {
                if (!mapValue.containsKey(l.getLinkTable().getName())) continue;
                Map<String, String> mapV = mapValue.get(l.getLinkTable().getName());

                String value = row.readString(l.getFieldName());
                if (StringUtils.isEmpty(value)) continue;

                String[] ids = StringUtils.split(value, ",");
                String names = "";
                for (String sId : ids) {
                    String sn = mapV.get(sId);
                    if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                }
                row.put(l.getFieldName() + "_text", names.substring(1));
            }
        }
    }
}
