package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelLinkName;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * Created by Akmm at 2022/3/3 20:04
 */
public class DefaultProvider<T extends DefaultEntity> extends AbstractCompProvider {
    public String tableName;

    public DefaultProvider(String tableName) {
        this.tableName = tableName;
    }

    public T getBean(long id) {
        return doGetData(tableName + "." + id, () -> this.loadBean(id));
    }

    private T loadBean(long id) {
        EntityDao<T> bdao = (EntityDao<T>) DbEngine.getInstance().findDao(tableName);
        T bean = bdao.queryEntity(id);
        //添加关联字段值
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table == null) return bean;
        List<ModelLinkName> listLink = table.findLinkeNames();
        if (listLink.isEmpty()) return bean;

        //有缓存的，从缓存拿，无缓存的放Map，最后从数据库去拿
        Map<String, List<String>> mapIds = new HashMap<>();

        for (ModelLinkName l : listLink) {
            String value = bean.getStr(l.getFieldName());
            if (StringUtils.isEmpty(value)) continue;
            String[] ids = StringUtils.split(value, ",");
            if (l.getLinkTable().isNeedCache()) {
                EntityDao dao = DbEngine.getInstance().findDao(l.getLinkTable().getName());
                AbstractCache cache = CacheManager.getIntance().getCache(l.getLinkTable().getName());
                String names = "";
                for (String sId : ids) {
                    Object b = cache.get(sId);
                    String sn = (String) dao.readValue(b, l.getLinkNameField());
                    if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                }
                bean.put(l.getFieldName() + "_text", names.substring(1));
            } else {
                List<String> list = mapIds.computeIfAbsent(l.getLinkTable().getName(), k -> new ArrayList<>());
                Collections.addAll(list, ids);
            }
        }


        if (mapIds.isEmpty()) return bean;
        //数据库查询
        Map<String, Map<String, String>> mapValue = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : mapIds.entrySet()) {
            EntityDao dao = DbEngine.getInstance().findDao(entry.getKey());
            mapValue.put(entry.getKey(), dao.queryNames(entry.getValue()));
        }
        //加值

        for (ModelLinkName l : listLink) {
            if (!mapValue.containsKey(l.getLinkTable().getName())) continue;
            Map<String, String> mapV = mapValue.get(l.getLinkTable().getName());

            String value = bean.getStr(l.getFieldName());
            if (StringUtils.isEmpty(value)) continue;

            String[] ids = StringUtils.split(value, ",");
            String names = "";
            for (String sId : ids) {
                String sn = mapV.get(sId);
                if (StringUtils.isNotEmpty(sn)) names += "," + sn;
            }
            bean.put(l.getFieldName() + "_text", names.substring(1));
        }
        return bean;
    }
}
