package cc.smtweb.framework.core.mvc.service;

import org.apache.commons.lang3.ArrayUtils;

//一个sql及其参数
public class SqlPara {
    public String sql = "";
    public Object[] paras = null;
    public int[] types = null;
    public boolean hasLob = false; //是否有lob字段

    public SqlPara(String sql, Object... paras) {
        this.sql = sql;
        this.paras = paras;
    }

    public SqlPara(String sql, int[] types, Object... paras) {
        this.sql = sql;
        this.paras = paras;
        this.types = types;
    }

    public SqlPara(String sql, int[] types, boolean hasLob, Object... paras) {
        this.sql = sql;
        this.paras = paras;
        this.types = types;
        this.hasLob = hasLob;
    }

    @Override
    public int hashCode() {
        //计算Hash值，以判断条件参数是否有变更；
        if (paras == null) return sql.hashCode();
        int result = 1;
        for (Object element : paras) {
            result = 31 * result + (element == null ? 0: element.hashCode());
        }
        result = 31 * result + sql.hashCode();
        return result;
    }

    //获取不带order by 的sql语句
    public String getSqlNoOrderBy() {
        String sSql = sql.toUpperCase();
        int n = sSql.lastIndexOf("ORDER BY ");//截取最后一个 order by
        if (n < 0) return sql;
        String retSql = sql.substring(0, n);
        String s = sSql.substring(n);
        if (s.contains("SELECT")) return sql; //最后一个order by之后，有子查询的不算
        if (!s.contains(")")) return retSql; //校验order by之后带括号的情况，一般是表达式
        n = 0;
        for (int i = 0, len = s.length(); i < len; i++) {
            final char c = s.charAt(i);
            if (c == '(') n++;
            else if (c == ')') n--;
            if (n < 0) return sql;
        }
        return retSql;
    }

    public void addParas(Object... ps) {
        if (paras == null) {
            paras = ps;
            return;
        }
        paras = ArrayUtils.addAll(paras, ps);
    }
}
