package cc.smtweb.system.bpm.core.ui.builder;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.sqlbuilder.AbstractSelectSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.AbstractUpdateSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.UpdateSqlBuilder;
import cc.smtweb.system.bpm.core.exception.BpmDataException;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.engine.ui.entity.consts.UiEnum;
import org.apache.commons.lang3.StringUtils;

public class CardTreeDataBuilder {
  private Long parentId;
  private String fieldParentId;
  private String fieldParentIdList;
  private String fieldLevel;
  private DbEngine dbEngine;

  public CardTreeDataBuilder(DbEngine dbEngine) {
    this.dbEngine = dbEngine;
  }

  public void add(int type, String fieldName, Object fieldValue) {
    switch (type) {
      case UiEnum.FieldRefType.PARENT_ID:
        parentId = readLong(fieldValue);
        fieldParentId = fieldName;
        break;
      case UiEnum.FieldRefType.PARENT_ID_LIST:
        fieldParentIdList = fieldName;
        break;
      case UiEnum.FieldRefType.TREE_LEVEL:
        fieldLevel = fieldName;
        break;
      default:
        throw new IllegalStateException("Unexpected value: " + type);
    }
  }

  public static Long readLong(Object s) {
    if (s != null) {
      if (s instanceof Number) {
        return ((Number) s).longValue();
      } else {
        String value = s.toString();
        if (StringUtils.isNotBlank(value)) {
          return Long.parseLong(value);
        }
      }
    }

    return null;
  }

  public AbstractUpdateSqlBuilder save(BpmDataset dataset, long dataId, AbstractUpdateSqlBuilder sqlBuilder) {
    if (fieldParentId != null) {
      if (fieldParentIdList == null && fieldLevel == null) {
        return null;
      }

      if (fieldParentIdList == null || fieldLevel == null) {
        throw new BpmDataException("必须同时设置parentIdList和level作用字段");
      }

      // TODO 检测需要同时设置idList和level
      String idFileName = dataset.readIdFieldName();
      String parentIdList = null;
      int treeLevel = 0;
      if (parentId == null) {
        sqlBuilder.add(fieldParentIdList, "");
        sqlBuilder.add(fieldLevel, 0);
        parentIdList = parentId + "-";
      } else {
        // 读取SQL
        AbstractSelectSqlBuilder select =  SqlBuilder.createSelect(dataset.getDatabase(), dataset.getTable());
        select.add(fieldParentIdList + " parentIdList");
        select.add(fieldLevel + " level");

        select.addWhere(idFileName, parentId);

        TreeItemVO parentVO = select.queryEntity(dbEngine, TreeItemVO.class);
        if (parentVO != null) {
          parentIdList = parentVO.getParentIdList() + parentId + "-";
          treeLevel = parentVO.getLevel() + 1;
          sqlBuilder.add(fieldParentIdList, parentIdList);
          sqlBuilder.add(fieldLevel, treeLevel);
        } else {
          throw new BpmDataException("not find parent by " + dataset.getTable() + "." + idFileName + "=" + parentId);
        }
      }

      // 如果parentId有修改，需要处理所有下级的 parentIdList 和 level
      if (dataId != 0) {
        AbstractSelectSqlBuilder select = SqlBuilder.createSelect(dataset.getDatabase(), dataset.getTable());
        select.add(fieldParentIdList, parentIdList);
        select.add(fieldParentId + " parentId");
        select.add(fieldLevel + " level");
        select.addWhere(idFileName, dataId);

        TreeItemVO curVO = select.queryEntity(dbEngine, TreeItemVO.class);
        if (curVO != null) {
          if (!LongEquals(curVO.getParentId(), parentId)) {
            parentIdList = parentIdList + dataId + "-";
            // update table set parentIdList=CONCAT('1-2-4-', SUBSTR(parentIdList, 6)) where parentIdList like "1-2-3-%", level = level + 1;
            UpdateSqlBuilder sb = SqlBuilder.createUpdate(dataset.getDatabase(), dataset.getTable());
            sb.add(String.format("%s=CONCAT('%s', SUBSTR(%s, %d))", fieldParentIdList, parentIdList, fieldParentIdList, curVO.getParentIdList().length() + 1),
                    UpdateSqlBuilder.VALUE_INVALID);
            if (curVO.getLevel() > treeLevel) {
              sb.add(String.format("%s=%s + %d", fieldLevel, fieldLevel, curVO.getLevel() - treeLevel), UpdateSqlBuilder.VALUE_INVALID);
            } else if (curVO.getLevel() < treeLevel) {
              sb.add(String.format("%s=%s - %d", fieldLevel, fieldLevel, treeLevel - curVO.getLevel()), UpdateSqlBuilder.VALUE_INVALID);
            }
            sb.addWhere(fieldParentIdList, curVO.getParentIdList() + "%", " like ");

            return sb;
          }
        } else {
          throw new BpmDataException("not find data by " + dataset.getTable() + "." + idFileName + "=" + dataId);
        }
      }
    }

    return null;
  }

  private static boolean LongEquals(Long oldValue, Long newValue) {
    if (oldValue != null) {
      return oldValue.equals(newValue);
    }

    return newValue == null;
  }
}
