package cc.smtweb.system.bpm.core.ui.builder;

import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;

import java.util.List;
import java.util.function.Function;

public class SqlUtil {
  // 包含
  static final int OP_SUB = 1;
  // 全匹配
  static final int OP_ALL = 2;
  // 前匹配
  static final int OP_BEGIN = 3;
  // 后匹配
  static final int OP_END = 4;
  // 等于
  static final int OP_EQUAL = 5;
  // 不等于
  static final int OP_NOT_EQUAL = 6;
  // 大于
  static final int OP_GRATER = 7;
  // 大于等于
  static final int OP_NOT_LESS = 8;
  // 小于
  static final int OP_LESS = 9;
  // 小于等于
  static final int OP_NOT_GRATER = 10;

  // 查询参数
  public static final String PN_FILTER = "filter";
  public static final String PN_SEARCH = "search";
  public static final String PN_SEARCH_TYPE = "searchType";

  private SqlUtil(){}

  public static void addSqlWhere(SqlBuilder sqlBuilder, String field, int op, Object keyword) {
    if (keyword instanceof String) {
      keyword = ((String) keyword).trim();
    }

    switch (op) {
      default:
      case OP_EQUAL:
      case OP_ALL:
        sqlBuilder.addWhere(field, keyword, "=");
        break;
      case OP_NOT_EQUAL:
        sqlBuilder.addWhere(field, keyword, "<>");
        break;
      case OP_GRATER:
        sqlBuilder.addWhere(field, keyword, ">");
        break;
      case OP_NOT_LESS:
        sqlBuilder.addWhere(field, keyword, ">=");
        break;
      case OP_LESS:
        sqlBuilder.addWhere(field, keyword, "<");
        break;
      case OP_NOT_GRATER:
        sqlBuilder.addWhere(field, keyword, "<=");
        break;
      case OP_SUB:
        sqlBuilder.addWhere(field, "%" + keyword + "%", " like ");
        break;
      case OP_BEGIN:
        sqlBuilder.addWhere(field, "%" + keyword, " like ");
        break;
      case OP_END:
        sqlBuilder.addWhere(field, keyword + "%", " like ");
        break;
    }
  }

  public static String makeSummaryField(String rule, String fieldName) {
    switch (rule) {
      case "${count}":
        return "count(" + fieldName + ")";
      case "${sum}":
        return "sum(" + fieldName + ")";
      case "${avg}":
        return "avg(" + fieldName + ")";
      case "${min}":
        return "min(" + fieldName + ")";
      case "${max}":
        return "max(" + fieldName + ")";
      default:
        return rule;
    }
  }

  public static <T> String join(List<T> list, Function<T, String> action) {
    if (list.isEmpty()) {
      return null;
    }

    StringBuilder sb = new StringBuilder();
    for (T t: list) {
      sb.append(action.apply(t)).append(',');
    }

    sb.setLength(sb.length() - 1);

    return sb.toString();
  }
}
