package cc.smtweb.system.bpm.core.ui.entity.dataset;

import cc.smtweb.system.bpm.engine.ui.entity.consts.UiEnum;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * BPM界面引擎后台执行用到的对象
 * @author xkliu
 */
@Data
public class BpmDataset {
  public static final int DATA_MODEL_CARD = 1;
  public static final int DATA_MODEL_LIST = 2;
  public static final int DATA_MODEL_TREE = 3;

  private String id;
  private String name;
  private String database;
  private String table;
  /** 主键字段名 */
  private BpmField idField;

  private String fieldPrefix;
  // 字段列表，name => UiField
  private Map<String, BpmField> fields = new HashMap<>();
  // 是否只读
  private boolean readonly;
  // 是否有统计字段
  private boolean useSummary;
  // 是否有查找字段
  private boolean useLookup;
  // 外键关联属性
  private AbstractBpmFieldLookup lookup;
  // 与主表的映射关系
  private BpmMasterMapping masterMapping = new BpmMasterMapping();
  // 1:1 映射的从表
  private List<BpmDataset> oneToOneDataset;
  // 数据模型
  private int dataModel;

  // 是否有级联数据集，TODO: 分离前台界面需要的参数
//  private boolean cascade;

  private String customApi;

  private boolean initData;

  private BpmDatasetSql sql;

  public void addField(BpmField bpmField) {
    fields.put(bpmField.getName(), bpmField);

    String summary = bpmField.getSummary();
    if (StringUtils.isNotBlank(summary)) {
      this.useSummary = true;
    }

    if (bpmField.getLookup() != null) {
      this.useLookup = true;
    }

    if (bpmField.getRefType() == UiEnum.FieldRefType.ID) {
      setIdField(bpmField);
//      props.setIdField(bpmField.getName());
    }
  }

  public BpmField addField(String fieldName, String name) {
    return addField(fieldName, name, 0);
  }

  public BpmField addField(String fieldName, String name, int refType) {
    BpmField bpmField = null;
    if (StringUtils.isNotBlank(fieldName)) {
      bpmField = new BpmField();
      bpmField.setId(name);
      bpmField.setFieldName(fieldName);
      bpmField.setName(name);
      bpmField.setRefType(refType);
      this.addField(bpmField);
    }

    return bpmField;
  }

  public BpmField findFieldByName(String name) {
    return fields.get(name);
  }

  public BpmField findFieldById(String fieldId) {
    return findFieldById(fieldId, false);
  }

  public BpmField findFieldById(String fieldId, boolean findSlave) {
    if (StringUtils.isNotBlank(fieldId)) {
      for (BpmField field : fields.values()) {
        if (fieldId.equals(field.getId())) {
          return field;
        }
      }
    }

    if(findSlave && this.oneToOneDataset != null) {
      for (BpmDataset slave: this.oneToOneDataset) {
        BpmField field = slave.findFieldById(fieldId);
        if (field != null) {
          return field;
        }
      }
    }

    return null;
  }

  public String fullName() {
    return database + "." + table;
  }

//  public String getFieldPrefix() {
//    return props.getFieldPrefix();
//  }
//
//  public String getId() {
//    return props.getId();
//  }

//  public BpmField findFieldByRefType(int refType) {
//    for (Map.Entry<String, BpmField> entry: fields.entrySet()) {
//      BpmField bpmField = entry.getValue();
//
//      if (bpmField.getRefType() == refType) {
//        return bpmField;
//      }
//    }
//
//    return null;
//  }

  public BpmField findFieldByFieldName(String fieldName) {
    if (fieldName != null) {
      for (Map.Entry<String, BpmField> entry : fields.entrySet()) {
        BpmField bpmField = entry.getValue();

        if (fieldName.equals(bpmField.getFieldName())) {
          return bpmField;
        }
      }
    }

    return null;
  }

  public void addOneToOneDataset(BpmDataset slave) {
    if (oneToOneDataset == null) {
      oneToOneDataset = new ArrayList<>();
    }

    oneToOneDataset.add(slave);
  }

//  public BpmDataset findById(String datasetId) {
//    if (datasetId.equals(getProps().getId())) {
//      return this;
//    }
//
//    if (this.oneToOneDataset != null) {
//      for (BpmDataset slave: this.oneToOneDataset) {
//        if (datasetId.equals(slave.getProps().getId())) {
//          return slave;
//        }
//      }
//    }
//
//    return null;
//  }

  public String readIdFieldName() {
    return idField != null ? idField.getFieldName() : null;
  }
}
