package cc.smtweb.system.bpm.core.ui.service.card;

import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.bind.IBpmBeanConst;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import cc.smtweb.system.bpm.core.ui.service.card.agent.DatasetLoadAgent;
import cc.smtweb.system.bpm.core.ui.service.card.agent.DatasetRemoveAgent;
import cc.smtweb.system.bpm.core.ui.service.card.agent.DatasetSaveAgent;

public class DatasetCardAgentWorker extends DatasetCardWorker {
  private DatasetSaveAgent datasetSaveAgent;
  private DatasetLoadAgent datasetLoadAgent;
  private DatasetRemoveAgent datasetRemoveAgent;

  public DatasetCardAgentWorker(BpmPageContext ctx) {
    super(ctx);
  }

  public void onBeforeSave(DatasetApiInvoker func) {
    loadDatasetSaveAgent(func.getBodyType()).setBeforeSave(func);
  }

  public void onAfterSave(DatasetApiInvoker func) {
    loadDatasetSaveAgent(func.getBodyType()).setAfterSave(func);
  }

  public void onSave(DatasetApiInvoker func) {
    loadDatasetSaveAgent(func.getBodyType()).setSave(func);
  }

  private DatasetSaveAgent loadDatasetSaveAgent(Class<?> clazz) {
    if (datasetSaveAgent == null) {
      datasetSaveAgent = new DatasetSaveAgent(saveFunc);
      this.handleSave(datasetSaveAgent::doSave);
    }

    datasetSaveAgent.checkType(clazz);

    return datasetSaveAgent;
  }

  public void onBeforeLoad(DatasetApiInvoker func) {
    loadDatasetLoadAgent(null).setBeforeLoad(func);
  }

  public void onLoad(DatasetApiInvoker func) {
    loadDatasetLoadAgent(func.getReturnType()).setLoad(func);
  }

  public void onAfterLoad(DatasetApiInvoker func) {
    loadDatasetLoadAgent(func.getAttrType(IBpmBeanConst.ATTRIBUTE_ROW)).setAfterLoad(func);
  }

  private DatasetLoadAgent loadDatasetLoadAgent(Class<?> clazz) {
    if (datasetLoadAgent == null) {
      datasetLoadAgent = new DatasetLoadAgent(loadFunc);
      this.handleLoad(datasetLoadAgent::doLoad);
    }

    datasetLoadAgent.checkType(clazz);

    return datasetLoadAgent;
  }

  public void onBeforeRemove(DatasetApiInvoker func) {
    loadDatasetRemoveAgent().setBeforeRemove(func);
  }

  public void onAfterRemove(DatasetApiInvoker func) {
    loadDatasetRemoveAgent().setAfterRemove(func);
  }

  public void onRemove(DatasetApiInvoker func) {
    loadDatasetRemoveAgent().setRemove(func);
  }

  private DatasetRemoveAgent loadDatasetRemoveAgent() {
    if (datasetRemoveAgent == null) {
      datasetRemoveAgent = new DatasetRemoveAgent(removeFunc);
      this.handleRemove(datasetRemoveAgent::doRemove);
      this.handleRemoveList(datasetRemoveAgent::doRemove);
    }

    return datasetRemoveAgent;
  }
}
