package cc.smtweb.system.bpm.core.ui.service.card;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldLookup;
import cc.smtweb.system.bpm.core.ui.service.DatasetWorker;

import java.util.function.BiFunction;
import java.util.function.Function;

public class DatasetCardWorker extends DatasetWorker {
  protected Function<BpmPageContext, SwMap> loadFunc;
  protected Function<BpmPageContext, Long> saveFunc;
  protected Function<BpmPageContext, Integer> removeFunc;
  protected Function<BpmPageContext, Integer> removeListFunc;
  protected BiFunction<BpmPageContext, Object, BpmKeyValue> loadLookupFunc;

  public DatasetCardWorker(BpmPageContext ctx) {
    super(ctx);
  }

  public void handleLoad(Function<BpmPageContext, SwMap> loadFunc) {
    this.loadFunc = loadFunc;
  }

  public void handleLoadLookup(BiFunction<BpmPageContext, Object, BpmKeyValue> loadLookupFunc) {
    this.loadLookupFunc = loadLookupFunc;
  }

  public void handleSave(Function<BpmPageContext, Long> saveFunc) {
    this.saveFunc = saveFunc;
  }

  public void handleRemove(Function<BpmPageContext, Integer> removeFunc) {
    this.removeFunc = removeFunc;
  }

  public void handleRemoveList(Function<BpmPageContext, Integer> removeListFunc) {
    this.removeListFunc = removeListFunc;
  }

  public SwMap load(BpmDataset dataset, long id) {
    pageContext.setDataset(dataset);
    pageContext.setFieldLookup(null);
    return loadFunc.apply(pageContext);
  }

  public BpmKeyValue loadLookup(BpmDataset dataset, BpmFieldLookup lookup, Object lookupKey) {
    pageContext.setDataset(dataset);
    pageContext.setFieldLookup(lookup);
    return loadLookupFunc.apply(pageContext, lookupKey);
  }

  public Long save() {
    return saveFunc.apply(pageContext);
  }

  public int remove(BpmDataset dataset, long dataId) {
    pageContext.setDataset(dataset);
    pageContext.setFieldLookup(null);
    return removeFunc.apply(pageContext);
  }

  public int removeList(BpmDataset dataset, String dataIds) {
    pageContext.setDataset(dataset);
    pageContext.setFieldLookup(null);
    return removeListFunc.apply(pageContext);
  }
}
