package cc.smtweb.system.bpm.core.ui.service.card.agent;

import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.service.AbstractDatasetAgent;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import java.util.function.Function;

/**
 * card数据集删除方法代理执行，调用@BpmEvent的事件
 * @author xkliu
 */
public class DatasetRemoveAgent extends AbstractDatasetAgent {
  @Setter
  private DatasetApiInvoker beforeRemove;
  @Setter
  private DatasetApiInvoker afterRemove;
  @Setter
  private DatasetApiInvoker remove;

  private final Function<BpmPageContext, Integer> defaultRemoveFunc;

  public DatasetRemoveAgent(Function<BpmPageContext, Integer> defaultRemoveFunc) {
    this.defaultRemoveFunc = defaultRemoveFunc;
  }

  public Integer doRemove(BpmPageContext pageContext) {
    Integer result = null;
    Long[] ids;
    String dataIds = pageContext.getBody().readString(IParamConst.PARAMS_IDS);
    if (StringUtils.isNotBlank(dataIds)) {
      ids = parse(dataIds);
    } else {
      ids = new Long[]{pageContext.getParamId()};
    }

    saveAttr(pageContext, "ids", ids);

    // bean 处理
    if (beforeRemove != null) {
      beforeRemove.invoke(pageContext);
    }

    if (this.remove != null) {
      result = (Integer)this.remove.invoke(pageContext);
    } else {
      result = this.defaultRemoveFunc.apply(pageContext);
    }

    if (afterRemove != null) {
      afterRemove.invoke(pageContext);
    }

    return result;
  }

  private Long[] parse(String ids) {
    String[] ary = ids.split(",");
    Long[] result = new Long[ary.length];
    for (int i = 0; i < result.length; i++) {
      result[i] = Long.valueOf(ary[i]);
    }

    return result;
  }
}
