package cc.smtweb.system.bpm.core.ui.service.list;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.system.bpm.core.ui.BpmListData;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatTool;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatter;
import cc.smtweb.system.bpm.core.ui.entity.dataset.*;
import cc.smtweb.system.bpm.core.ui.service.AbstractDatasetService;
import cc.smtweb.system.bpm.engine.ui.entity.consts.UiEnum;

import java.util.*;

/**
 * 列表数据集抽象类
 * @author xkliu
 */
public abstract class AbstractDatasetListService extends AbstractDatasetService {
  /**
   * 创建列表数据集ListPart处理器
   * @param pageContext 页面上下文参数
   * @return 列表数据集处理器
   */
  protected abstract DatasetListWorker createListPartHandler(BpmPageContext pageContext);
  /**
   * 创建列表数据集ListPart处理器
   * @param pageContext 页面上下文参数
   * @return 列表数据集处理器
   */
  protected abstract DatasetListWorker createListAllHandler(BpmPageContext pageContext);

  public R list(BpmPageContext pageContext) {
    DatasetListWorker handler = createListPartHandler(pageContext);
//    setupListPart(handler);
    BpmListData data = doList(handler, false);
    return R.success(data);
  }

  private BpmListData doList(DatasetListWorker handler, boolean listAll) {
    BpmDataset dataset = handler.getDataset();

//    IDynFormList dynFormList = uiPage.isDelpoy() ? dynFormTableList : dynFormJsonList;
    List<SwMap> list = null;
    boolean loadCount = false;

    AbstractBpmFieldLookup lookup = dataset.getLookup();
    if (lookup != null) {
      // 读取字典表
      if (lookup.type() == UiEnum.FieldLookupType.DICT) {
        BpmFieldDict fieldDict = (BpmFieldDict) lookup;
        // 字典数据
        /*Map<String, DictPO> dictMap = dbCache.dictMap(DictPO.class, fieldDict.getDictType());

        if (dictMap != null) {
          List<SwMap> dictList = new ArrayList<>(dictMap.size());

          for (DictPO dictPo : dictMap.values()) {
            SwMap item = new SwMap();

            item.put(IParamConst.OPTION_LABEL, dictPo.getDictLabel());
            if (lookup.isNumber()) {
              item.put(IParamConst.OPTION_KEY, Integer.parseInt(dictPo.getDictCode()));
            } else {
              item.put(IParamConst.OPTION_KEY, dictPo.getDictCode());
            }

            dictList.add(item);
          }

          list = dictList;
        }*/
      } else if (lookup.type() == UiEnum.FieldLookupType.TABLE) {
        list = handler.listAll(dataset, (BpmFieldLookup)lookup);
      }
    } else {
      // TODO 减少无需要显示的字段，同时减少lookup
      if (listAll) {
        list = handler.listAll(dataset, null);
      } else {
        list = handler.list(dataset);
        if (list != null) {
          if (dataset.isUseSummary()) {
            loadCount = true;
          } else {
            int limit = handler.getPageContext().getBody().readInt(IParamConst.PARAM_PAGE_SIZE, IParamConst.PAGE_DEFAULT_SIZE);
            if (list.size() >= limit) {
              loadCount = true;
            }
          }
        }
      }

      // 统一返回列表数据 data[datasetId].list.data
      if (list != null) {
        // lookupField 查询
        if (!list.isEmpty()) {
          listLookup(handler, dataset, list);

          if (dataset.getOneToOneDataset() != null) {
            for (BpmDataset slave : dataset.getOneToOneDataset()) {
              listLookup(handler, slave, list);
            }
          }
        }
      }
    }

    // TODO -1表示需要异步获取count

    return  BpmListData.create(list, loadCount);
  }

  private Map<String, String> listDict(BpmFieldDict lookup, Set<String> ids) {
    // 字典数据
    /*Map<String, DictPO> dictMap = dbCache.dictMap(DictPO.class, lookup.getDictType());

    if (dictMap != null) {
      Map<String, String> result = new HashMap<>(ids.size());

      for (String id : ids) {
        DictPO dictPo = dictMap.get(id);

        if (dictPo != null) {
          result.put(id, dictPo.getDictLabel());
        }
      }

      return result;
    }*/

    return null;
  }

  private void listLookup(DatasetListWorker context, BpmDataset dataset, List<SwMap> list) {
    List<FieldFormatter> fieldFormatters = null;

    // 查找lookup表数据，并初始化格式化器
    for (BpmField bpmField : dataset.getFields().values()) {
      AbstractBpmFieldLookup lookup = bpmField.getLookup();
      if (lookup != null) {
        String uiFieldName = bpmField.getName();
        // TODO 处理N:N非ID字段情况
        if (bpmField == dataset.getIdField()) {
          uiFieldName = IParamConst.PARAM_ID;
        }

        // 读取查找表数据
        Set<String> keys = new HashSet<>();
        for (SwMap item : list) {
          Object key = item.get(uiFieldName);
          if (key != null) {
            keys.add(key.toString());
          }
        }

        if (!keys.isEmpty()) {
          Map<String, String> lookValues = null;
          if (lookup.type() == UiEnum.FieldLookupType.TABLE) {
            lookValues = context.listLookup(dataset, (BpmFieldLookup)lookup, keys);
          } else if (lookup.type() == UiEnum.FieldLookupType.DICT) {
            lookValues = listDict((BpmFieldDict) lookup, keys);
          }

          if (lookValues != null) {
            String textKey = bpmField.getName() + IParamConst.LOOKUP_TEXT;
            for (SwMap item : list) {
              Object key = item.get(uiFieldName);
              if (key != null) {
                String value = lookValues.get(key.toString());
                if (value != null) {
                  item.put(textKey, value);
                }
              }
            }
          }
        }
      } else if(FieldFormatTool.needFormat(bpmField)) {
        if (fieldFormatters == null) {
          fieldFormatters = new ArrayList<>();
        }

        fieldFormatters.add(FieldFormatTool.buildFormatter(bpmField));
      }
    }

    // 格式化字段显示
    if (fieldFormatters != null) {
      for (SwMap item : list) {
        for (FieldFormatter fieldFormatter: fieldFormatters) {
          Object value = item.get(fieldFormatter.getName());
          if (value != null) {
            item.put(fieldFormatter.getName() + IParamConst.LOOKUP_TEXT, fieldFormatter.format(value));
          }
        }
      }
    }
  }

  public R count(BpmPageContext pageContext) {
    DatasetListWorker handler = createListPartHandler(pageContext);
//    setupListPart(handler);
    BpmDataset dataset = handler.getDataset();

    SwMap data = handler.count(dataset);
    return R.success(data);
  }

  public R listAll(BpmPageContext pageContext) {
    DatasetListWorker handler = createListAllHandler(pageContext);
//    setupListAll(handler);
    BpmListData data = doList(handler, true);
    return R.success(data);
  }
}
