package cc.smtweb.system.bpm.engine;

import cc.smtweb.system.bpm.core.exception.ModelLoaderError;
import lombok.Getter;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public abstract class AbstractLoader<T> {
  /** 错误信息 */
  @Getter
  protected List<ModelLoaderError> errors = new ArrayList<>();

  protected void clearErrors() {
    this.errors.clear();
  }

  public void addError(int type, String msg) {
    this.errors.add(new ModelLoaderError(type, msg));
  }

  public void addError(int type, String id, String msg) {
    this.errors.add(new ModelLoaderError(type, id, msg));
  }

  public void addError(int type, String id, String detailId, String msg) {
    this.errors.add(new ModelLoaderError(type, id, detailId, msg));
  }

  /** 根据map转换String为int值 */
  protected int mapValue(Map<String, Integer> map, String orginValue, int defaultValue) {
    Integer value = map.get(orginValue);
    return (value != null) ? value : defaultValue;
  }

  protected int mapValue(Map<String, Integer> map, String orginValue) {
    return mapValue(map, orginValue, 0);
  }

  public String getLastError() {
    if (this.errors.isEmpty()) {
      return null;
    }

    return this.errors.get(0).getMsg();
  }

  public abstract <T> T load(String jsonText);

//  public <T> T load(InputStream is) {
//    return null;
//  }
}
