package cc.smtweb.system.bpm.engine.process.runtime;

import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.core.exception.BpmObjectNotFoundException;
import cc.smtweb.system.bpm.engine.process.vo.Flow;
import cc.smtweb.system.bpm.engine.process.vo.FlowAction;
import cc.smtweb.system.bpm.engine.process.vo.FlowConnection;
import cc.smtweb.system.bpm.engine.process.vo.FlowNode;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

public class ProcessInstance extends ProcessInfo {
  @Getter
  private Flow flow;

  @Getter
  private ModelTable modelTable;

  @Getter
  private FlowNode curNode;

  @Getter @Setter
  private ConditionExpression conditionExpression;

  @Getter
  private ProcessRoute processRoute = new ProcessRoute();

  public ProcessInstance(long id, Flow flow,  ModelTable modelTable) {
    this.id = id;
    this.flow = flow;
    this.modelTable = modelTable;
  }

  /** 添加乐观锁计数，在修改记录成功后调用 */
  public void incLockRev() {
    this.lockRev++;
  }

  public FlowNode nextNode(int actionType) {
    if (curNode != null) {
      List<FlowAction> actions = curNode.getActions();
      if (actions != null) {
        for (FlowAction action: actions) {
          if (action.getType() == actionType) {
            FlowConnection connection = action.getConnection();

            if (connection != null) {
              String nodeId = connection.getToNodeId();
              return this.flow.findNode(nodeId);
            }

            break;
          }
        }
      }
    }

    return null;
  }

  public FlowAction checkAndGetCurNodeAction(String actionId) {
    if (curNode == null) {
      throw new BpmObjectNotFoundException("current node is null");
    }

    return curNode.checkAndGetAction(actionId);
  }

  public FlowNode nextNode(String actionId) {
    if (curNode != null) {
      List<FlowAction> actions = curNode.getActions();
      if (actions != null) {
        for (FlowAction action: actions) {
          if (actionId.equals(action.getId())) {
            FlowConnection connection = action.getConnection();

            if (connection != null) {
              String nodeId = connection.getToNodeId();
              return this.flow.findNode(nodeId);
            }

            break;
          }
        }
      }
    }

    return null;
  }

  public void setCurNode(FlowNode flowNode, boolean addRoute) {
    this.curNode = flowNode;
    if (addRoute) {
      this.processRoute.setNode(flowNode);
    }
  }

  public FlowNode nextNode(FlowAction flowAction) {
    FlowConnection connection = flowAction.getConnection();

    if (connection != null) {
      String nodeId = connection.getToNodeId();
      return this.flow.findNode(nodeId);
    }

    return null;
  }

  public ProcessInstance createSubProcess(long id, FlowNode flowNode) {
    ProcessInstance result = new ProcessInstance(id, flow, modelTable);

    result.assign(this);
    result.setParentId(this.getId());
    result.curNode = flowNode;
    result.conditionExpression = conditionExpression;

    // 添加当前路由节点
    this.processRoute.addSubProcess(result);

    return result;
  }

  public String postRoute() {
    ProcessRouteInfo info = this.processRoute.post();

    if (info != null) {
      return JsonUtil.encodeString(info);
    }

    return null;
  }

  public void fillRoute(String processRoute) {
    if (StringUtils.isNotBlank(processRoute)) {
      this.processRoute.setInfo(JsonUtil.parse(processRoute, ProcessRouteInfo.class));
    }
  }

  public ProcessInstance createSame(long id) {
    ProcessInstance result = new ProcessInstance(id, flow, modelTable);

    result.assign(this);
    result.setParentId(this.getId());
    result.conditionExpression = conditionExpression;
    result.processRoute = this.processRoute;

    return result;
  }

  // 从日志恢复数据，新建流程
  public void resume(long newId) {
    this.id = newId;
    this.setCreateAt(0L);
    this.setUpdateAt(0L);
  }
}
