package cc.smtweb.system.bpm.engine.process.vo;

import cc.smtweb.system.bpm.core.exception.BpmObjectNotFoundException;
import lombok.Data;

import java.util.List;

/**
 * @author xkliu
 * 流程的活动节点对象
 */
@Data
public class FlowNode {
  /** 开始 */
  public static final int TYPE_BEGIN = 10;
  /** 结束 */
  public static final int TYPE_END = 20;
  /** 任务 */
  public static final int TYPE_MANUAL = 30;
  /** 会签 */
  public static final int TYPE_MULTI_SIGN = 40;
  /** 条件 */
  public static final int TYPE_CONDITION = 50;
  /** 分支 */
  public static final int TYPE_SPLIT = 60;
  /** 聚合 */
  public static final int TYPE_MERGE = 70;
  /** 子流程 */
  public static final int TYPE_SUB_FLOW = 80;
  /** 自动 */
  public static final int TYPE_AUTO = 90;

  /** 活动类型，枚举值参考TYPE_XXX */
  private int type;

  private String id;
  /** 标题 */
  private String label;
  /** 英文名称 */
  private String name;
  /** 禁止提醒 */
  private String disableNotify;
  /** 字段权限列表 */
  private List<FlowNodeField> fields;
  /** 执行人列表 */
  private List<FlowNodeUser> users;
  /** 动作列表 */
  private List<FlowAction> actions;

  public FlowAction findAction(int typeCommit) {
    for (FlowAction flowAction: actions) {
      if (flowAction.getType() == typeCommit) {
        return flowAction;
      }
    }

    return null;
  }

  public FlowAction checkAndGetAction(String actionKey) {
    FlowAction nextFlowAction = findAction(actionKey);

    if (nextFlowAction == null) {
      throw new BpmObjectNotFoundException("未找到Action, action_key=" + actionKey);
    }

    return nextFlowAction;
  }

  private FlowAction findAction(String actionKey) {
    for (FlowAction flowAction: actions) {
      if (flowAction.getId().equals(actionKey)) {
        return flowAction;
      }
    }

    return null;
  }
}
