package cc.smtweb.system.bpm.engine.ui.codegen;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;

import java.io.IOException;
import java.security.MessageDigest;

public class Sha256Helper {
  // 是否跳过空字符（\r\n\t )和（逗号，分号，单引号，双引号），避免格式化后不能生成代码
  private boolean skipSpaceChar;
  private MessageDigest md = DigestUtils.getSha256Digest();
  private int size;
  public static final byte[] SPACE_CHAR = new byte[256];

  static {
    SPACE_CHAR[' '] = 1;
    SPACE_CHAR['\r'] = 2;
    SPACE_CHAR['\n'] = 3;
    SPACE_CHAR['\t'] = 1;
    SPACE_CHAR[','] = 1;
    SPACE_CHAR[';'] = 1;
    SPACE_CHAR['\''] = 1;
    SPACE_CHAR['"'] = 1;
  }

  public Sha256Helper(boolean skipSpaceChar) {
    this.skipSpaceChar = skipSpaceChar;
  }

  public void write(int b) throws IOException {
    if (md != null) {
      if (skipSpaceChar) {
        digitUpdate(md, (byte) b);
      } else {
        md.update((byte) b);
        size ++;
      }
    }
  }

  public void write(byte[] b) {
    write(b, 0, b.length);
  }

  public void write(byte[] b, int off, int len) {
    if (md != null) {
      if (skipSpaceChar) {
        for (int i = off; i < off + len; i++) {
          digitUpdate(md, b[i]);
        }
      } else {
        md.update(b, off, len);
        size += len;
      }
    }
  }

  private void digitUpdate(MessageDigest md, byte ch) {
    //  != ' ' && ch != '\r' && ch != '\n' && ch != '\t' && ch != ',' && ch != ';' && ch != '\'' && ch != '"'
    if (ch < 0 || SPACE_CHAR[ch] == 0) {
      md.update(ch);
      size++;
    }
  }

  public String sha256Hex() {
    String reuslt = null;
    if (md != null) {
      reuslt = Hex.encodeHexString(md.digest());
      md = null;
    }

    return reuslt;
  }

  public int digestSize() {
    return size;
  }
}
