package cc.smtweb.system.bpm.engine.ui.codegen;

import java.io.IOException;
import java.io.OutputStream;

public class Sha256OutputStream extends OutputStream {
  private final OutputStream os;
  // 是否跳过空字符和逗号（\r\n\t )，避免格式化后不能生成代码
  private Sha256Helper md;

  public Sha256OutputStream(OutputStream os, boolean skipSpaceChar) {
    this.os = os;
    this.md = new Sha256Helper(skipSpaceChar);
  }

  @Override
  public void write(int b) throws IOException {
    os.write(b);
    md.write(b);
  }

  @Override
  public void write(byte[] b, int off, int len) throws IOException {
    os.write(b, off, len);
    md.write(b, off, len);
  }

  @Override
  public void flush() throws IOException {
    os.flush();
  }

  @Override
  public void close() throws IOException {
    os.close();
  }

  public String sha256Hex() {
    return md.sha256Hex();
  }

  public int digestSize() {
    return md.digestSize();
  }
}
