package cc.smtweb.system.bpm.engine.ui.codegen;

import cc.smtweb.framework.core.util.jackson.*;
import cc.smtweb.system.bpm.util.CodeGenUtil;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.json.JsonWriteFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.json.JsonMapper;
import com.fasterxml.jackson.databind.module.SimpleModule;
import org.apache.commons.lang3.StringUtils;

import java.sql.Timestamp;

/**
 * Velocity生成用的工具函数
 * @author xkliu
 */
public class VmTool {
  private final ObjectMapper OBJECT_MAPPER = JsonMapper.builder()
          .disable(JsonWriteFeature.QUOTE_FIELD_NAMES).build();

  private final ObjectMapper OBJECT_MAPPER_FULL = JsonMapper.builder()
          .disable(JsonWriteFeature.QUOTE_FIELD_NAMES).build();

  VmTool() {
    OBJECT_MAPPER.setSerializationInclusion(JsonInclude.Include.NON_NULL);
    OBJECT_MAPPER.setSerializationInclusion(JsonInclude.Include.NON_EMPTY);
    OBJECT_MAPPER.setSerializationInclusion(JsonInclude.Include.NON_DEFAULT);
    initObjectMapper(OBJECT_MAPPER);

    OBJECT_MAPPER_FULL.setSerializationInclusion(JsonInclude.Include.ALWAYS);
    initObjectMapper(OBJECT_MAPPER_FULL);
  }

  private void initObjectMapper(ObjectMapper objectMapper) {
    SimpleModule module = new SimpleModule();
    LongSerializer longSerializer = new LongSerializer();
    module.addSerializer(Long.class, longSerializer);
    module.addSerializer(Long.TYPE, longSerializer);

    DateSerializer dateSerializer = new DateSerializer();
    module.addSerializer(java.sql.Date.class, dateSerializer);
    TimeSerializer timeSerializer = new TimeSerializer();
    module.addSerializer(java.sql.Time.class, timeSerializer);

    module.addDeserializer(Timestamp.class, new TimestampDeserializer());
    module.addDeserializer(java.util.Date.class, new DateDeserializer());

    objectMapper.registerModule(module);
  }

  public String json(Object obj) throws JsonProcessingException {
    return OBJECT_MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(obj);
  }

  public String jsonFull(Object obj) throws JsonProcessingException {
    return OBJECT_MAPPER_FULL.writerWithDefaultPrettyPrinter().writeValueAsString(obj);
  }

  public String gridArea(int x, int y, int w, int h) {
    return (y + 1) + "/" + (x + 1) + "/ span " + h + "/span " + w;
  }

  public int length(Object[] list) {
    if (list != null) {
      return list.length;
    }

    return 0;
  }

  public String toHump(String value) {
    return CodeGenUtil.underlineToHump(value);
  }

  public String toUpperHump(String value) {
    return CodeGenUtil.underlineToUpperHump(value);
  }

  /**
   * 生成模板属性，根据不同的值类型进行处理
   */
  public String htmlProp(String key, Object value) throws JsonProcessingException {
    if (value != null) {
      String name = CodeGenUtil.humpToDash(key);
      if (value instanceof Boolean) {
        if (Boolean.TRUE.equals(value)) {
          return name;
        }
        return "";
      } else if (value instanceof Number) {
        return ":" + name + "=\"" + value + "\"";
      } else if (value instanceof String) {
        if (StringUtils.isNotBlank((String)value)) {
          return name + "=\"" + value + "\"";
        }
      } else {
        return ":" + name + "=\"" + escapeJson(OBJECT_MAPPER.writeValueAsString(value)) + "\"";
      }
    }

    return "";
  }

  private String escapeJson(String value) {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < value.length(); i++) {
      char ch = value.charAt(i);
      if (ch == '\"') {
        sb.append('\'');
      } else if (ch == '\'') {
        sb.append("\\'");
      } else {
        sb.append(ch);
      }
    }

    return sb.toString();
  }
}
