package cc.smtweb.system.bpm.engine.ui.loader.json;

import cc.smtweb.framework.core.db.cache.ModelDatabaseCache;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelDatabase;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.util.MapUtil;
import cc.smtweb.system.bpm.core.exception.ModelLoaderError;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatTool;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.*;
import cc.smtweb.system.bpm.engine.AbstractLoader;
import cc.smtweb.system.bpm.engine.ui.entity.consts.MappingTypeEnum;
import cc.smtweb.system.bpm.engine.ui.entity.consts.UiEnum;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzDatasetPropsVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzDatasetVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzFieldVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzMasterMappingVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.widiget.UiControlPropsVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.widiget.UiPageVO;
import cc.smtweb.system.bpm.engine.ui.loader.yaml.YamlDatasetDecoder;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class JsonDatasetDecoder {
  private AbstractLoader loader;
  private Map<String, SlaveDataset> datasetMap = new HashMap<>();
  private Map<String, ModelDatabase> databases = new HashMap<>();
  @Getter
  private List<SlaveDataset> slaveDatasetList = new ArrayList<>();
//  private Map<BpmField, BpmDataset> lookupDatasetMap = new HashMap<>();

  public JsonDatasetDecoder(AbstractLoader loader) {
    this.loader = loader;
  }

  protected void decode(BpmPage result, UiPageVO uiPageVO) {
    // 处理DataSet
    buildDataset(uiPageVO.getModel(), result, slaveDatasetList);

    fixMasterDataset(result, slaveDatasetList);
  }

  private String getMasterName(BpmDataset dataset) {
//    BpmMasterMapping bpmMasterMapping = dataset.getMasterMapping();
//    if (bpmMasterMapping != null && bpmMasterMapping.isOneToOne()) {
//      return bpmMasterMapping.getMaster();
//    }

    return dataset.getName();
  }

//  private void addLookupField(BpmDataset dataset, String fieldName, String name, int refType) {
//    if (StringUtils.isNotBlank(fieldName)) {
//      BpmField bpmField = new BpmField();
//      bpmField.setFieldName(fieldName);
//      bpmField.setName(name);
//      bpmField.setRefType(refType);
//      dataset.addField(bpmField);
//    }
//  }

  private void fixMasterDataset(BpmPage result, List<SlaveDataset> slaveDatasetList) {
//    if (result.getMasterDataset() != null) {
//      UiPageFormDataset pageFormDataset = result.getMasterDataset().getPageFormDataset();
//      if (pageFormDataset.getTableType() != UiPageFormDataset.TABLE_TYPE_MASTER) {
//        pageFormDataset.setTableType(UiPageFormDataset.TABLE_TYPE_MASTER);
//      }
//    }

    for (SlaveDataset slaveDataset: slaveDatasetList) {
      BpmDataset slave = slaveDataset.getDataset();
//      UiPageFormDataset slaveProps = slaveDataset.getPageFormDataset();
      String masterId = slaveDataset.getMasterId();
      BpmDataset master = findDataset(masterId);

      if (master != null) {
        // 1:1 映射需要重新设置从表名称
        FzMasterMappingVO masterMappingVO = slaveDataset.getMasterMapping();
        BpmDataset middle = null;

        BpmMasterMapping masterMapping = slave.getMasterMapping();

        masterMapping.setMappingType(MappingTypeEnum.toEnum(masterMappingVO.getMappingType()));
        masterMapping.setMaster(master.getName());

        // 处理从表映射
        if (MappingTypeEnum.oneToOne.equals(masterMapping.getMappingType())) {
          // 名字改成一样，便于控件引用
//          slaveProps.setName(master.getName());
          slave.setName(master.getName());
          master.addOneToOneDataset(slave);
        } else if (MappingTypeEnum.manyToMany.equals(masterMapping.getMappingType())) {
          String middleId = masterMappingVO.getMiddleDataset();
          if (StringUtils.isBlank(middleId)) {
            loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未设置Middle数据集");
            continue;
          }

          middle = findDataset(middleId);
          if (middle == null) {
            loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未找到Middle数据集, id=" + masterId);
            continue;
          }

          masterMapping.setMiddle(middle.getName());
        }

        // 处理字段映射
        for (BpmMappingField linkFieldVO: masterMappingVO.getFields()) {
          BpmField masterField = master.findFieldById(linkFieldVO.getMaster());

          if (masterField == null) {
            loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未找到Master数据集字段, id=" + linkFieldVO.getMaster());
            continue;
          }

          BpmField slaveField = slave.findFieldById(linkFieldVO.getSlave());
          if (slaveField == null) {
            loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未找到Slave数据集字段, id=" + linkFieldVO.getSlave());
            continue;
          }

          String middleFieldName = null;
          if (middle != null) {
            BpmField middleField = middle.findFieldById(linkFieldVO.getMiddle());
            if (middleField == null) {
              loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未找到Middle数据集字段, id=" + linkFieldVO.getMiddle());
              continue;
            }

            middleFieldName = middleField.getFieldName();
          }

          masterMapping.add(masterField.getFieldName(), slaveField.getFieldName(), middleFieldName);
        }
      } else {
        loader.addError(ModelLoaderError.ERROR_UI_DATASET, slave.getId(), "未找到Master数据集, id=" + masterId);
      }
    }
  }

  public BpmDataset findDataset(String masterId) {
    BpmDataset result = null;
    SlaveDataset linkDataset = datasetMap.get(masterId);
    if (linkDataset == null) {
      for (SlaveDataset slaveDataset : slaveDatasetList) {
        if (masterId.equals(slaveDataset.getDataset().getId())) {
          result = slaveDataset.getDataset();
          break;
        }
      }
    } else {
      result = linkDataset.getDataset();
    }

    return result;
  }

  private void buildDataset(FzDatasetVO[] fzDatasetAry, BpmPage result, List<SlaveDataset> slaveDatasetList) {
    if (fzDatasetAry != null) {
      for (FzDatasetVO datasetVO : fzDatasetAry) {
        FzDatasetPropsVO datasetPropsVO = datasetVO.getProps();

        if (datasetPropsVO.getSql() != null) {
          buildBpmDataset(result, slaveDatasetList, datasetVO, null, null);
        } else {
          if (StringUtils.isBlank(datasetPropsVO.getDatasource())) {
            loader.addError(ModelLoaderError.ERROR_DB, "未设置数据库");
          } else {
            String datasource = datasetPropsVO.getDatasource();

            ModelDatabase db = getModelDatabase(datasource);

            if (db == null) {
              loader.addError(ModelLoaderError.ERROR_DB, "未找到数据库, name= " + datasource);
            } else {
              ModelTable mobileTable = ModelTableCache.getInstance().getByName(datasetPropsVO.getTable());

              if (mobileTable == null) {
                loader.addError(ModelLoaderError.ERROR_TABLE, "未找到数据表 " + datasetPropsVO.getTable() + ", datasource: " + datasource);
              } else {
                buildBpmDataset(result, slaveDatasetList, datasetVO, db, mobileTable);
              }
            }
          }
        }
      }
    }
  }

  private void buildBpmDataset(BpmPage result, List<SlaveDataset> slaveDatasetList, FzDatasetVO datasetVO, ModelDatabase db, ModelTable mobileTable) {
    boolean addToPage = true;
    FzDatasetPropsVO datasetPropsVO = datasetVO.getProps();
    BpmDataset dataset = new BpmDataset();

    dataset.setId(datasetVO.getId());
    dataset.setDataModel(toInt(datasetPropsVO.getDataModel()));
    dataset.setInitData(datasetPropsVO.isInitData());
    dataset.setCustomApi(StringUtils.trimToNull(datasetPropsVO.getCustomApi()));

    if (db != null) {
      dataset.setDatabase(db.getName());
      dataset.setTable(mobileTable.getName());
      dataset.setFieldPrefix((YamlDatasetDecoder.getFieldPrefix(mobileTable.getIdField())));
    } else {
      dataset.setSql(datasetPropsVO.getSql());
    }

    if (StringUtils.isNotBlank(datasetPropsVO.getName())) {
      dataset.setName(datasetPropsVO.getName());
    } else {
      dataset.setName(datasetVO.getId());
    }

    SlaveDataset linkDataset = new SlaveDataset(dataset, datasetVO.getId(), datasetPropsVO.getMasterDataset(), datasetPropsVO.getMasterFields());
    if (linkDataset.isSlave()) {
      slaveDatasetList.add(linkDataset);
      if (MappingTypeEnum.oneToOne.sameName(datasetPropsVO.getMasterFields().getMappingType())) {
        addToPage = false;
      }
    }

    FzFieldVO[] fields = datasetPropsVO.getFields();

    for (FzFieldVO fieldVO : fields) {
      BpmField bpmField = new BpmField();
      UiControlPropsVO fieldProps = fieldVO.getProps();
      String field = fieldProps.readString("field");

      if (mobileTable != null) {
        ModelField modelField = mobileTable.findField(field);

        if (StringUtils.isNotBlank(field) && modelField == null) {
          loader.addError(ModelLoaderError.ERROR_FIELD, "未找到字段 " + dataset.getTable() + "." + field);
          continue;
        }

        // 处理自动名，虚拟数据集用ID作为字段名
        if (modelField != null) {
          if (StringUtils.isNotBlank(modelField.getName())) {
            bpmField.setFieldName(modelField.getName());
          } else {
            // 测试时可以使用ID来作为字段名
            bpmField.setFieldName(modelField.getName());
          }
        }
      } else {
        bpmField.setFieldName(field);
      }

      bpmField.setId(fieldVO.getId());
      bpmField.setSummary(fieldProps.readString("summary"));
      bpmField.setRefType(fieldProps.readInt("refType", 0));
      bpmField.setRequired(fieldProps.readBool("required", false));
      bpmField.setValidPatten(fieldProps.readString("validPatten"));
      bpmField.setValidMessage(fieldProps.readString("validMessage"));
      bpmField.setLabel(fieldProps.readString("label"));
      bpmField.setDisplayFormat(fieldProps.readString("displayFormat"));
      bpmField.setFilter(fieldProps.readInt("filter", 0));
      bpmField.setFilterType(fieldProps.readInt("filterType", 0));

      // 处理查找属性
      Map<String, Object> lookup = fieldProps.readMap("lookup");
      if (lookup != null) {
        fixFieldLookup(datasetVO, fieldVO, bpmField, lookup);
      }

      // 处理变量名，控件关联
      String name = fieldProps.readString("name");
      if (StringUtils.isNotBlank(name)) {
        bpmField.setName(name);
      } else {
        bpmField.setName(bpmField.getId());
      }

      String fieldType = fieldVO.getType();
      if (fieldType != null && fieldType.startsWith("fz-field-")) {
        bpmField.setType(fieldType.substring(9));
      } else {
        loader.addError(ModelLoaderError.ERROR_FIELD, datasetVO.getId(), fieldVO.getId(), "无效字段类型 " + fieldType);
      }

      FieldFormatTool.fixDisplayFormat(bpmField);
      dataset.addField(bpmField);
    }

    if (dataset.getIdField() == null) {
      if (mobileTable != null) {
        BpmField idField = dataset.findFieldByFieldName(mobileTable.getIdField());
        dataset.setIdField(idField);
      }

      if (dataset.getIdField() == null) {
        loader.addError(ModelLoaderError.ERROR_TABLE, "数据表未设置ID字段 " + datasetPropsVO.getTable() + ", dataset: " + dataset.getName());
        return;
      }
    }

//              datasetProps.setIdField(dataset.getIdField().getName());

    if (addToPage) {
      addDataset(result, linkDataset);
    }
  }

  private boolean fixFieldLookup(FzDatasetVO datasetVO, FzFieldVO fieldVO, BpmField bpmField, Map<String, Object> lookup) {
    String type = MapUtil.readString(lookup, "type");
    String dbName = MapUtil.readString(lookup, "db");
    String table = MapUtil.readString(lookup, "table");
    boolean isNumber = isNumberType(fieldVO.getType());

    if ("dict".equals(type)) {
      String dictType = MapUtil.readString(lookup, "dictType");

      if (StringUtils.isBlank(dictType)) {
        loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置dictType");
        return true;
      }

      bpmField.setLookup(new BpmFieldDict(dbName, table, isNumber, dictType));
    } else if ("table".equals(type) || "lookup".equals(type)) { // "lookup" 是兼容旧配置
      MappingTypeEnum mappingType = MappingTypeEnum.toEnum(MapUtil.readString(lookup, "mappingType"));
      if (!mappingType.include(MappingTypeEnum.manyToOne, MappingTypeEnum.manyToMany)) {
        loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置映射类型");
        return true;
      }

      // 数据库字段名
      String keyField = MapUtil.readString(lookup, "keyField");
      if (StringUtils.isBlank(keyField)) {
        loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置keyField");
        return true;
      }

      // 数据库字段名
      String labelField = MapUtil.readString(lookup, "labelField");
      if (StringUtils.isBlank(labelField)) {
        loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置labelField");
        return true;
      }

      BpmFieldLookup fieldLookup = new BpmFieldLookup(dbName, table, isNumber, mappingType);

      fieldLookup.setKeyField(keyField);
      fieldLookup.setLabelField(labelField);

      // 数据库上级字段名
      String parentField = MapUtil.readString(lookup, "parentField");
      if (StringUtils.isNotBlank(parentField)) {
        fieldLookup.setParentField(parentField);
      }

      List<Map<String, Object>> filters = (List<Map<String, Object>>)lookup.get("filter");
      if (filters != null && !filters.isEmpty()) {
        for (Map<String, Object> filter: filters) {
          String filterField = MapUtil.readString(filter, IBpmDatasetConst.PN_ITEM_FIELD);
          String filterKeyword = MapUtil.readString(filter, IBpmDatasetConst.PN_ITEM_KEYWORD);
          int filterOp = MapUtil.readInt(filter, IBpmDatasetConst.PN_ITEM_OP, 0);
          boolean cascade = MapUtil.readBool(filter, IBpmDatasetConst.PN_ITEM_CASCADE, false);

          fieldLookup.addFilter(filterField, filterOp, filterKeyword, cascade);
        }
      }

      if (mappingType.equals(MappingTypeEnum.manyToMany)) {
        // 数据库字段名
        String middleTable = MapUtil.readString(lookup, "middleTable");
        if (StringUtils.isBlank(middleTable)) {
          loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置中间表");
          return true;
        }
        fieldLookup.setMiddleTable(middleTable);

        String middleKeyField = MapUtil.readString(lookup, "middleKeyField");
        if (StringUtils.isBlank(middleKeyField)) {
          loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置中间表映射字段");
          return true;
        }
        fieldLookup.setMiddleKeyField(middleKeyField);

        String middleValueField = MapUtil.readString(lookup, "middleValueField");
        if (StringUtils.isBlank(middleValueField)) {
          loader.addError(ModelLoaderError.ERROR_UI_DATASET, datasetVO.getId(), fieldVO.getId(), "lookup未配置中间表映射字段");
          return true;
        }
        fieldLookup.setMiddleValueField(middleValueField);
      }

      bpmField.setLookup(fieldLookup);
    } // else "dataset"

    return false;
  }

  private void addDataset(BpmPage result, SlaveDataset slaveDataset) {
    List<BpmDataset> datasets = result.getDatasets();
    int insert = -1;

    String id = slaveDataset.getId();
    if (id != null) {
      // 保证主表在前
      for (int i = 0; i < datasets.size(); i++) {
        SlaveDataset linkDataset = datasetMap.get(datasets.get(i).getId());

        if (linkDataset != null) {
          if (id.equals(linkDataset.getMasterId())) {
            insert = i;
            break;
          }
        }
      }

      datasetMap.put(slaveDataset.getId(), slaveDataset);
    }

    BpmDataset dataset = slaveDataset.getDataset();
    if (insert >= 0) {
      datasets.add(insert, dataset);
    } else {
      datasets.add(dataset);
    }
  }

  public void addLookupDataset(BpmPage result, BpmDataset masterDataset, BpmField field, boolean isTree) {
    // 1:1关联需要找到主表名称
    String masterName = getMasterName(masterDataset);

    String name = masterName + "_" + field.getName();

    BpmDataset datasetWrapper = result.findDatasetByName(name);
    if (datasetWrapper == null) {
      AbstractBpmFieldLookup lookup = field.getLookup();

      ModelDatabase db = getModelDatabase(lookup.getDb());
      if (db == null) {
        loader.addError(ModelLoaderError.ERROR_UI, "未找到数据源, database= " + lookup.getDb());
        return;
      }

      BpmDataset dataset = new BpmDataset();

      dataset.setInitData(true);
      dataset.setName(name);
      dataset.setDatabase(lookup.getDb());
      dataset.setTable(lookup.getTable());
      dataset.setLookup(lookup);
      dataset.setDataModel(isTree ? BpmDataset.DATA_MODEL_TREE : BpmDataset.DATA_MODEL_LIST);

      if (lookup instanceof BpmFieldLookup) {
        BpmFieldLookup fieldLookup = (BpmFieldLookup)lookup;

        dataset.setFieldPrefix(YamlDatasetDecoder.getFieldPrefix(fieldLookup.getKeyField()));

        ModelTable modelTable = ModelTableCache.getInstance().getByName(lookup.getTable());

        if (modelTable == null) {
          loader.addError(ModelLoaderError.ERROR_UI, "未找到数据表, table= " + lookup.getDb() + "." + lookup.getTable());
          return;
        }

        dataset.addField(modelTable.getIdField(), "optId", UiEnum.FieldRefType.ID);
        dataset.addField(fieldLookup.getKeyField(), IParamConst.OPTION_KEY);
        dataset.addField(fieldLookup.getLabelField(), IParamConst.OPTION_LABEL, UiEnum.FieldRefType.TITLE);
        dataset.addField(fieldLookup.getOrderField(), IParamConst.OPTION_ORDER, UiEnum.FieldRefType.ORDER);
        dataset.setFieldPrefix(YamlDatasetDecoder.getFieldPrefix(modelTable.getIdField()));

        if (isTree) {
          if (fieldLookup.getParentField() == null) {
            loader.addError(ModelLoaderError.ERROR_UI, "树控件未配置上级字段, field= " + field.getName());
            return;
          }

          dataset.addField(fieldLookup.getParentField(), "optParentId", UiEnum.FieldRefType.PARENT_ID);
        }

        if (fieldLookup.isManyToMany()) {
          modelTable = ModelTableCache.getInstance().getByName(fieldLookup.getMiddleTable());

          if (modelTable == null) {
            loader.addError(ModelLoaderError.ERROR_UI, "未找到中间数据表, table= " + lookup.getDb() + "." + lookup.getTable());
            return;
          }

          fieldLookup.setMiddleIdField(modelTable.getIdField());
        }

        // 级联属性设置，不初始化，与级联库关联（master）
        if (fieldLookup.getFilters() != null) {
          for (BpmFieldFilter bpmFieldFilter: fieldLookup.getFilters()) {
            if (bpmFieldFilter.isCascade()) {
//              pageRef.addCascade(bpmDataset.getMasterMapping().getMaster(), bpmFieldFilter.getKeyword(), bpmDataset.getName(), bpmFieldFilter.getField());
//              dataset.setCascade(true);
              dataset.getMasterMapping().setMaster(masterName);
              dataset.setInitData(false);
            }
          }
        }

        if (dataset.getIdField() == null) {
          loader.addError(ModelLoaderError.ERROR_TABLE, "数据表未设置ID字段 " + dataset.getTable() + ", db: " + dataset.getDatabase());
        }
      }

      addDataset(result, new SlaveDataset(dataset, null, null, null));
    } else {
      int dataModel = isTree ? BpmDataset.DATA_MODEL_TREE : BpmDataset.DATA_MODEL_LIST;
      if (datasetWrapper.getDataModel() != dataModel) {
        loader.addError(ModelLoaderError.ERROR_UI, "多控件关联数据集时数据模型冲突 " + name);
      }
    }
  }

  private boolean isNumberType(String type) {
    return "fz-field-int".equals(type) || "fz-field-short".equals(type) || "fz-field-byte".equals(type);
  }

  private int toInt(Integer value) {
    return value != null ? value : 0;
  }

  private ModelDatabase getModelDatabase(String datasource) {
    ModelDatabase result = databases.get(datasource);

    if(result != null) {
      return result;
    }

    return ModelDatabaseCache.getInstance().get(datasource);
  }

//  public void addLookupDataset(BpmPage result, BpmField bpmField, boolean isTree) {
//    BpmDataset dataset = this.lookupDatasetMap.get(bpmField);
//    if (dataset != null) {
//      // 查找页面路由
//      addLookupDataset(result, dataset, bpmField, isTree);
//    } else {
//      loader.addError(ModelLoaderError.ERROR_UI, "未找到lookup数据集, field= " + bpmField.getName());
//    }
//  }
}

