package cc.smtweb.system.bpm.engine.ui.loader.json;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.core.exception.ModelLoaderError;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatTool;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.AbstractBpmFieldLookup;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmField;
import cc.smtweb.system.bpm.core.ui.entity.dataset.IBpmDatasetConst;
import cc.smtweb.system.bpm.core.ui.entity.form.BpmForm;
import cc.smtweb.system.bpm.core.ui.entity.form.BpmWidget;
import cc.smtweb.system.bpm.core.ui.entity.form.BpmWidgetLayout;
import cc.smtweb.system.bpm.engine.AbstractLoader;
import cc.smtweb.system.bpm.engine.ui.entity.vo.widiget.*;
import cc.smtweb.system.bpm.engine.ui.loader.IPageRouter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;

@Slf4j
public class JsonPageDecoder {
  private static final int LIST_MODEL_SELECT = 1000;
  private static final int LIST_MODEL_SELECT_TREE = IBpmDatasetConst.DATA_MODEL_TREE + LIST_MODEL_SELECT;
  private static final int LIST_MODEL_SELECT_LIST = IBpmDatasetConst.DATA_MODEL_LIST + LIST_MODEL_SELECT;

  private AbstractLoader loader;
  private IPageRouter pageRouter;
  private JsonDatasetDecoder jsonDatasetDecoder;

  private final static Map<String, Integer> CONTROL_LIST_MODEL = new HashMap<String, Integer>() {{
    put("fx-list", IBpmDatasetConst.DATA_MODEL_LIST);
    put("fx-table", IBpmDatasetConst.DATA_MODEL_LIST);
    put("fy-list", IBpmDatasetConst.DATA_MODEL_LIST);
    put("fy-table", IBpmDatasetConst.DATA_MODEL_LIST);
    put("fx-tree", IBpmDatasetConst.DATA_MODEL_TREE);
    put("fy-tree", IBpmDatasetConst.DATA_MODEL_TREE);
    put("fx-select", LIST_MODEL_SELECT);
    put("fy-select", LIST_MODEL_SELECT);
    put("fx-select-tree", LIST_MODEL_SELECT_TREE);
    put("fy-select-tree", LIST_MODEL_SELECT_TREE);
    put("fx-select-table", LIST_MODEL_SELECT_LIST);
  }};

  public JsonPageDecoder(AbstractLoader loader, IPageRouter pageRouter) {
    this.loader = loader;
    this.pageRouter = pageRouter;
    jsonDatasetDecoder = new JsonDatasetDecoder(loader);
  }

  public BpmPage decode(String content) {
    UiPageVO uiPageVO = JsonUtil.parse(content, UiPageVO.class);

    if (uiPageVO.getVersion() == null || uiPageVO.getVersion() < 5) {
      log.error("界面版本不能小于 5.0; version=" + uiPageVO.getVersion());
      return new BpmPage();
    }

    BpmPage result = new BpmPage();

    // 处理DataSet
    jsonDatasetDecoder.decode(result, uiPageVO);

    // 处理界面控件
    UiPagePropsVO pageProps = null;
    for (UiPageFormVO formVO: uiPageVO.getForms()) {
      BpmForm form = new BpmForm();
      fixUiForm(result, form, formVO.getGraph());
      result.getForms().add(form);

      // 处理主界面熟悉
      if (pageProps == null) {
        pageProps = formVO.getWidget().getProps();
        if (pageProps != null) {
          result.setDestroyOnClose(pageProps.getDestroyOnClose());
        }
      }
    }
    return result;
  }

  /** 处理界面控件 */
  private void fixUiForm(BpmPage result, BpmWidget widget, UiControlVO parent) {
    /*
     控件进行dataset, 通用规则 data[dataset][dataModel][field]
     卡片界面控件设置 data[datasetId].form[fieldId]
     列表控件设置 data[datasetId].list.data
    */
    UiControlDataVO controlData = parent.getData();
    // 校正数据，设置数据集
    BpmDataset dataset = fixContrData(controlData);

    widget.setName(controlData.getName());
    widget.setType(controlData.getType());
    widget.addProps(controlData.getProps());
    widget.addSlots(controlData.getSlots());
    widget.addVars(controlData.getVars());
    widget.addEvents(controlData.getEvents());

    if (dataset != null) {
      fixWidget(result, widget, dataset, controlData);
    }

    // 设置布局尺寸
    UiControlLayoutVO layout = controlData.getLayout();
    if (layout != null && layout.getCol() != null) {
      widget.setLayout(new BpmWidgetLayout(layout.getCol()));
    }

    //    String datasetId = props.readString("dataset");
//    if (StringUtils.isNotBlank(datasetId)) {
//      BpmDataset dataset = jsonDatasetDecoder.findDataset(datasetId);
//
//      if (dataset != null) {
//        fixWidget(result, widget, dataset, controlData);
//
//        // 转换 id 为 name
//        widget.addProp("dataset", dataset.getName());
//      } else {
//        loader.addError(ModelLoaderError.ERROR_UI, controlData.getId(),"控件关联数据集未找到, datasetId= " + datasetId);
//      }
//    }

    // 处理panel的子控件
    UiControlVO[] children = parent.getChildren();
    if (children != null &&  children.length > 0) {
      BpmWidget[] widgetChildren = new BpmWidget[children.length];
      for (int i = 0; i < children.length; i++) {
        BpmWidget childWidget = new BpmWidget();
        fixUiForm(result, childWidget, children[i]);
        widgetChildren[i] = childWidget;
      }

      widget.setChildren(widgetChildren);
    }

    // 处理插槽的控件
    if (widget.getSlots() != null) {
      for (UiControlDataVO[] slotWidgets: widget.getSlots().values()) {
        for (UiControlDataVO slotWidget: slotWidgets) {
          fixContrData(slotWidget);
          if (dataset != null) {
            fixSlotWidget(slotWidget, dataset, controlData.getId());
          }
        }
      }
    }
  }

  // 设置table.column的显示字段
  private void fixSlotWidget(UiControlDataVO controlWidget, BpmDataset dataset, String widgetId) {
    SwMap props = controlWidget.getProps();
    String field = props.readString("field");
    if (StringUtils.isNotBlank(field)) {
      BpmField bpmField = dataset.findFieldById(field, true);
      if (bpmField != null) {
        String fieldName = bpmField.getName();
        if (StringUtils.isBlank(fieldName)) {
          fieldName = bpmField.getId();
        }

        // 切换到显示字段
        if (bpmField.getLookup() != null || FieldFormatTool.needFormat(bpmField)) {
          props.put("field", fieldName + IParamConst.LOOKUP_TEXT);
        } else {
          props.put("field", fieldName);
        }

//              fixWidgetFormat(widget, map, bpmField);
      } else {
        loader.addError(ModelLoaderError.ERROR_UI, widgetId,
                controlWidget.getId(), "未找到表格控件的列字段 " + field);
      }
    }
  }

  // 预处理控件属性，包括slot控件
  private BpmDataset fixContrData(UiControlDataVO controlData) {
    SwMap props = controlData.getProps();
    // 保持name不为空
    String name = props.readString("name");
    if (StringUtils.isNotBlank(name)) {
      controlData.setName(name);
    } else {
      controlData.setName(controlData.getId());
    }

    // 处理button链接地址
    if (controlData.getType().endsWith("-button")) {
      String link = (String) props.get("link");
      if (StringUtils.isNotBlank(link)) {
        Long pageId = Long.valueOf(link);
        // 查找页面路由
        String routerName = pageRouter.loadFullName(pageId);
        if (routerName != null) {
          props.put("link", routerName);
        } else {
          loader.addError(ModelLoaderError.ERROR_UI, "控件关联页面未找到, pageId= " + pageId);
        }
//      pageRef.getRoutes().put("id_" + pageId, routerName);
      }
    }

    // 处理数据集，转换id为name
    String datasetId = props.readString("dataset");
    if (StringUtils.isNotBlank(datasetId)) {
      BpmDataset dataset = jsonDatasetDecoder.findDataset(datasetId);

      if (dataset != null) {
        // 转换 id 为 name
        props.put("dataset", dataset.getName());
      } else {
        loader.addError(ModelLoaderError.ERROR_UI, controlData.getId(),"控件关联数据集未找到, datasetId= " + datasetId);
      }
      return dataset;
    }

    return null;
  }

  // 主控件检测数据模型和处理lookup
  private void fixWidget(BpmPage result, BpmWidget widget, BpmDataset dataset, UiControlDataVO controlData) {
    UiControlPropsVO controlProps = controlData.getProps();
    String field = controlProps.readString("field");

    int listModel = getListMode(controlData.getType(), field);

    if (listModel == IBpmDatasetConst.DATA_MODEL_LIST) {
      if (dataset.getDataModel() != IBpmDatasetConst.DATA_MODEL_LIST) {
        loader.addError(ModelLoaderError.ERROR_UI, controlData.getId(), "控件需要引用列表模式数据集 " + dataset.getName());
      }

      // 1. field id => name, 2. fix lookup field
//      fixColumnField(widget, dataset, readList(widget.getProps(), "columns"), controlData.getId());
    } else if (listModel == IBpmDatasetConst.DATA_MODEL_TREE) {
      if (dataset.getDataModel() != IBpmDatasetConst.DATA_MODEL_TREE) {
        loader.addError(ModelLoaderError.ERROR_UI, controlData.getId(), "控件需要引用树模式数据集 " + dataset.getName());
      }
    } else {
      if (StringUtils.isNotBlank(field)) {
        BpmField bpmField = dataset.findFieldById(field);

        if (bpmField != null) {
          // 处理关联表
          AbstractBpmFieldLookup lookup = bpmField.getLookup();

          if (lookup != null) {
//            controlProps.put("lookup", lookup);
            if (listModel >= LIST_MODEL_SELECT) { //  || listModel == LIST_MODEL_SELECT_TREE
              // 设置使用了查找数据集，后面进行实际添加
              jsonDatasetDecoder.addLookupDataset(result, dataset, bpmField, listModel == LIST_MODEL_SELECT_TREE);
            }
          }

          widget.addProp("field", bpmField.getName());
//          fixWidgetFormat(widget, controlProps, bpmField);
//          datasetProps.addRule(bpmField.getName(), validType, required, validPatten, validMessage);
        } else {
          loader.addError(ModelLoaderError.ERROR_UI, controlData.getId(), "控件未找到字段, id= " + field);
        }
      }
    }

    // 处理 button 的链接地址
//    fixPageRef(widget);
  }

  private int getListMode(String controlType, String field) {
    Integer type = CONTROL_LIST_MODEL.get(controlType);

    if (type != null) {
      if (StringUtils.isBlank(field) || type >= LIST_MODEL_SELECT) {
        return type;
      } else {
        return type + LIST_MODEL_SELECT;
      }
    }

    return IBpmDatasetConst.DATA_MODEL_CARD;
  }

  /** 完善table列field，切换到显示字段 */
//  private void fixColumnField(BpmWidget widget, BpmDataset dataset, List columns, String widgetId) {
//    if (columns != null) {
//      for (Object item: columns) {
//        if (item instanceof Map) {
//          Map<String, Object> column = (Map<String, Object>) item;
//          Map<String, Object> map = (Map<String, Object>)column.get("props");
//
//          String field = MapUtil.readString(map, "field");
//          if (StringUtils.isNotBlank(field)) {
//            BpmField bpmField = dataset.findFieldById(field, true);
//            if (bpmField != null) {
//              String fieldName = bpmField.getName();
//              if (StringUtils.isBlank(fieldName)) {
//                fieldName = bpmField.getId();
//              }
//
//              // 切换到显示字段
//              if (bpmField.getLookup() != null || FieldFormatTool.needFormat(bpmField)) {
//                map.put("field", fieldName + IParamConst.LOOKUP_TEXT);
//              } else {
//                map.put("field", fieldName);
//              }
//
//            } else {
//              loader.addError(ModelLoaderError.ERROR_UI, widgetId,
//                      MapUtil.readString(column, "id"), "未找到表格控件的列字段 " + field);
//            }
//          }
//        }
//      }
//    }
//  }

//  public List readList(SwMap map, String name) {
//    Object v = map.get(name);
//
//    if (v instanceof List) {
//      return (List)v;
//    }
//
//    return null;
//  }

  // pageId -> 转化为pagePath
//  private void fixPageRef(BpmWidget widget) {
//    SwMap controlPropsVO = widget.getProps();
//
//    switch (widget.getType()) {
//      case "fx-table":
//      case "fy-table":
//        fillButtons(controlPropsVO, "columnButtons");
//        fillButtons(controlPropsVO, "toolbarButtons");
//        // table.columnButtons, table.toolbarButtons
//        break;
//      case "fx-button-group":
//      case "fz-button-group":
//        fillButtons(controlPropsVO, "buttons");
//        // button-group.buttons
//        break;
//      case "fx-button":
//      case "fz-button":
//        fillButton(controlPropsVO);
//        break;
//      default:
//        break;
//    }
//  }
//
//  private void fillButtons(SwMap controlPropsVO, String name) {
//    Object props = controlPropsVO.get(name);
//    if (props instanceof List) {
//      for (Object prop: (List)props) {
//        if (prop instanceof Map) {
//          Object controlProps = ((Map)prop).get("props");
//          if (controlProps instanceof Map) {
//            fillButton((Map<String, Object>) controlProps);
//          }
//        }
//      }
////      log.error("" + props.getClass());
//    }
//  }
//
//  private void fillButton(Map<String, Object> controlPropsVO) {
//    String link = (String)controlPropsVO.get("link");
//    if (StringUtils.isNotBlank(link)) {
//      Long pageId = Long.valueOf(link);
//      // 查找页面路由
//      String routerName = pageRouter.loadFullName(pageId);
//      if(routerName != null) {
//        controlPropsVO.put("link", routerName);
//      } else {
//        loader.addError(ModelLoaderError.ERROR_UI, "控件关联页面未找到, pageId= " + pageId);
//      }
////      pageRef.getRoutes().put("id_" + pageId, routerName);
//    }
//  }
}

