package cc.smtweb.system.bpm.spring.action.service;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import cc.smtweb.system.bpm.core.ui.service.card.AbstractDatasetCardService;
import cc.smtweb.system.bpm.core.ui.service.card.DatasetCardAgentWorker;
import cc.smtweb.system.bpm.core.ui.service.card.DatasetCardWorker;
import cc.smtweb.system.bpm.spring.dao.DatasetCardDao;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;
import lombok.Setter;

/**
 * 动态界面Card数据服务
 * @author xkliu
 */
public class BpmActionCardService extends AbstractDatasetCardService {
  protected DatasetCardDao datasetCardDao;
  // 动态数据集
  private String dataset;
  // 检测是否重复设置
//  @Setter
//  protected BiFunction<BpmPageContext, Long, SwMap> loadFunc;
//  @Setter
//  protected BiFunction<BpmPageContext, SwMap, Long> saveFunc;
//  @Setter
//  protected BiFunction<BpmPageContext, Long, Integer> removeFunc;
//  @Setter
//  protected BiFunction<BpmPageContext, String, Integer> removeListFunc;
//  @Setter
//  protected BiFunction<BpmPageContext, Object, BpmKeyValue> loadLookupFunc;

  @Setter
  private DatasetApiInvoker beforeLoad;
  @Setter
  private DatasetApiInvoker afterLoad;
  @Setter
  private DatasetApiInvoker load;
  @Setter
  private DatasetApiInvoker beforeSave;
  @Setter
  private DatasetApiInvoker afterSave;
  @Setter
  private DatasetApiInvoker save;
  @Setter
  private DatasetApiInvoker beforeRemove;
  @Setter
  private DatasetApiInvoker afterRemove;
  @Setter
  private DatasetApiInvoker remove;

  public BpmActionCardService(DbEngine dbEngine, DatasetConfigDao datasetConfigDao, DatasetCardDao datasetCardDao, String dataset) {
    this.dataset = dataset;
    super.init(dbEngine, datasetConfigDao);
    this.datasetCardDao = datasetCardDao;
  }

  @Override
  protected DatasetCardWorker createCardLoadHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);
    DatasetCardAgentWorker worker = new DatasetCardAgentWorker(pageContext);

    worker.handleLoad(datasetCardDao::load);
    worker.handleLoadLookup(datasetCardDao::loadLookup);

    if (beforeLoad != null) {
      worker.onBeforeLoad(beforeLoad);
    }

    if (afterLoad != null) {
      worker.onAfterLoad(afterLoad);
    }

    if (load != null) {
      worker.onLoad(load);
    }

    return worker;
  }

  @Override
  protected DatasetCardWorker createCardSaveHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);
    DatasetCardAgentWorker worker = new DatasetCardAgentWorker(pageContext);
    worker.handleSave(datasetCardDao::save);

    if (beforeSave != null) {
      worker.onBeforeSave(beforeSave);
    }

    if (afterSave != null) {
      worker.onAfterSave(afterSave);
    }

    if (save != null) {
      worker.onSave(save);
    }

    return worker;
  }

  @Override
  protected DatasetCardWorker createCardRemoveHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);
    DatasetCardAgentWorker worker = new DatasetCardAgentWorker(pageContext);

    worker.handleRemove(datasetCardDao::remove);

    worker.onBeforeRemove(beforeRemove);
    worker.onAfterRemove(afterRemove);
    worker.onRemove(remove);

    return worker;
  }
}
