package cc.smtweb.system.bpm.spring.action.service;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import cc.smtweb.system.bpm.core.ui.service.list.AbstractDatasetListService;
import cc.smtweb.system.bpm.core.ui.service.list.DatasetListAgentWorker;
import cc.smtweb.system.bpm.core.ui.service.list.DatasetListWorker;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;
import cc.smtweb.system.bpm.spring.dao.DatasetListDao;
import lombok.Setter;

/**
 * 动态界面List数据服务
 * @author xkliu
 */
public class BpmActionListService extends AbstractDatasetListService {
  protected DatasetListDao datasetListDao;
  private String dataset;

  @Setter
  private DatasetApiInvoker beforeCount;
  @Setter
  private DatasetApiInvoker afterCount;
  @Setter
  private DatasetApiInvoker count;

  @Setter
  private DatasetApiInvoker beforeList;
  @Setter
  private DatasetApiInvoker list;
  @Setter
  private DatasetApiInvoker afterList;

  @Setter
  private DatasetApiInvoker listAll;

  public BpmActionListService(DbEngine dbEngine, DatasetConfigDao datasetConfigDao, DatasetListDao datasetListDao, String dataset) {
    this.dataset = dataset;
    super.init(dbEngine, datasetConfigDao);
    this.datasetListDao = datasetListDao;
  }

  @Override
  protected DatasetListWorker createListPartHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);

    DatasetListAgentWorker worker = new DatasetListAgentWorker(pageContext);

    worker.handleListPart(datasetListDao::list);
    worker.handleCount(datasetListDao::count);
    worker.handleListLookup(datasetListDao::listLookup);

    if (beforeCount != null) {
      worker.onBeforeCount(beforeCount);
    }
    if (afterCount != null) {
      worker.onAfterCount(afterCount);
    }
    if (count != null) {
      worker.onCount(count);
    }
    if (beforeList != null) {
      worker.onBeforeList(beforeList);
    }
    if (afterList != null) {
      worker.onAfterList(afterList);
    }
    if (list != null) {
      worker.onListPart(list);
    }

    return worker;
  }

  @Override
  protected DatasetListWorker createListAllHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);

    DatasetListAgentWorker worker = new DatasetListAgentWorker(pageContext);

    worker.handleListAll(datasetListDao::listAll);
    worker.handleListLookup(datasetListDao::listLookup);

    if (beforeList != null) {
      worker.onBeforeList(beforeList);
    }

    if (afterList != null) {
      worker.onAfterList(afterList);
    }

    if (listAll != null) {
      worker.onListAll(listAll);
    }

    return worker;
  }
}
