package cc.smtweb.system.bpm.spring.action.service;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import cc.smtweb.system.bpm.core.ui.service.tree.AbstractDatasetTreeService;
import cc.smtweb.system.bpm.core.ui.service.tree.DatasetTreeAgentWorker;
import cc.smtweb.system.bpm.core.ui.service.tree.DatasetTreeWorker;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;
import cc.smtweb.system.bpm.spring.dao.DatasetTreeDao;
import lombok.Setter;

/**
 * 动态界面List数据服务
 * @author xkliu
 */
public class BpmActionTreeService extends AbstractDatasetTreeService {
  protected DatasetTreeDao datasetTreeDao;
  private String dataset;

  @Setter
  private DatasetApiInvoker beforeMove;
  @Setter
  private DatasetApiInvoker afterMove;
  @Setter
  private DatasetApiInvoker move;
  @Setter
  private DatasetApiInvoker beforeTreeAll;
  @Setter
  private DatasetApiInvoker treeAll;
  @Setter
  private DatasetApiInvoker afterTreeAll;

  public BpmActionTreeService(DbEngine dbEngine, DatasetConfigDao datasetConfigDao, DatasetTreeDao datasetTreeDao, String dataset) {
    this.dataset = dataset;
    super.init(dbEngine, datasetConfigDao);
    this.datasetTreeDao = datasetTreeDao;
  }

  @Override
  protected DatasetTreeWorker createTreeAllHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);

    DatasetTreeAgentWorker worker = new DatasetTreeAgentWorker(pageContext);
    worker.handleTreeAll(datasetTreeDao::treeAll);

    if (beforeTreeAll != null) {
      worker.onBeforeTreeAll(beforeTreeAll);
    }
    if (afterTreeAll != null) {
      worker.onAfterTreeAll(afterTreeAll);
    }
    if (treeAll != null) {
      worker.onTreeAll(treeAll);
    }

    return worker;
  }

  @Override
  protected DatasetTreeWorker createTreeMoveHandler(BpmPageContext pageContext) {
    datasetConfigDao.loadDataset(pageContext, dataset);

    DatasetTreeAgentWorker worker = new DatasetTreeAgentWorker(pageContext);
    worker.handleMove(datasetTreeDao::move);

    if (beforeMove != null) {
      worker.onBeforeMove(beforeMove);
    }
    if (afterMove != null) {
      worker.onAfterMove(afterMove);
    }
    if (move != null) {
      worker.onMove(move);
    }

    return worker;
  }
}
