package cc.smtweb.system.bpm.spring.cache;

import cc.smtweb.framework.core.annotation.SwCache;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.engine.entity.AspModelPO;
import cc.smtweb.system.bpm.engine.process.loader.FlowLoader;
import cc.smtweb.system.bpm.engine.process.vo.Flow;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

@SwCache(ident = "BPM_FLOW", title = "工作流定义")
@Slf4j
public class BpmFlowCache extends AbstractCache<Flow> {
  @Autowired
  private DbEngine dbEngine;

  public static BpmFlowCache getInstance() {
      return CacheManager.getIntance().getCache(BpmFlowCache.class);
  }

  @Override
  protected String getId(Flow bean) {
    return String.valueOf(bean.getId());
  }

  @Override
  protected List<Flow> loadAll() {
    String sql = "select model_id, model_mc_id, model_site_id, model_name, model_status, model_content, model_type, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
            " where model_type=?";
    List<AspModelPO> list = dbEngine.query(sql, AspModelPO.class, 2);
    List<Flow> listRet = new ArrayList<>(list.size());
    for (AspModelPO po: list) {
        FlowLoader flowLoader = new FlowLoader();
        Flow flow = flowLoader.load(po.getModelContent());

        if (flow == null) {
            log.error(flowLoader.getLastError());
        } else {
            listRet.add(flow);
        }
    }
    return listRet;
  }
}
