package cc.smtweb.system.bpm.spring.cache;

import cc.smtweb.framework.core.annotation.SwCache;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.engine.entity.AspModelPO;
import cc.smtweb.system.bpm.engine.entity.AspModelTreeVO;
import cc.smtweb.system.bpm.engine.ui.loader.*;
import cc.smtweb.system.bpm.spring.BpmConfigBean;
import cc.smtweb.system.bpm.util.ITreeDataLevelHandler;
import cc.smtweb.system.bpm.util.TreeDataUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

@SwCache(ident = "BPM_PAGE", title = "页面设计", lazy = true)
@Slf4j
public class BpmPageCache extends AbstractCache<BpmPage> {
    private final static String fn = "n";
    @Autowired
    private DbEngine dbEngine;

    @Autowired
    private BpmConfigBean bpmConfigBean;

    public static  BpmPageCache getInstance() {
        return CacheManager.getIntance().getCache(BpmPageCache.class);
    }

    public BpmPageCache() {
        regMap(fn, BpmPage::getFullName);
    }

    @Override
    protected String getId(BpmPage bean) {
        return String.valueOf(bean.getId());
    }

    @Override
    protected List<BpmPage> loadAll() {
        return null;
    }

    protected BpmPage load(Long key) {
        BpmPage result = null;

        if (key != null) {
            IPageRouter pageRouter = null;
            if (bpmConfigBean.isDebug()) {
                // 设计界面界面数据
                // json内容
                String sql = "select model_id, model_mc_id, model_parent_id, model_site_id, model_key, model_name, model_status, model_content, model_type, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
                        " where model_id=? and model_type in (3,4)";
                AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, key);

                // 项目树
                ITreeDataLevelHandler<AspModelTreeVO> handler = AspModelTreeVO.createTreeHandler();

                List<AspModelTreeVO> list = dbEngine.query("select model_id, model_parent_id, model_key, model_name from sw_bpm.asp_model where model_mc_id=?",
                        AspModelTreeVO.class, po.getModelMcId());
                AspModelTreeVO root = new AspModelTreeVO();
                TreeDataUtil.buildLevelTree(root, list, handler);
                String moduleName = dbEngine.queryString("select mc_module from sw_bpm.asp_model_catalog where mc_id=?", po.getModelMcId());

                pageRouter = new TreePageRouter(root, moduleName);
                result = loadFromJson(po, pageRouter);
            } else {
                // 部署界面数据
                String sql = "select model_id, model_site_id, model_full_name, model_name, model_status, model_content, model_type, model_version, model_create_time, model_last_time from sw_bpm.bpm_model" +
                        " where model_id=? and model_type in (3,4)";
                AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, key);

                if (po != null) {
                    // TODO 路由缓存性能
                    sql = "select model_id, model_site_id, model_full_name, model_name, model_status, model_type from sw_bpm.bpm_model" +
                            " where model_type=? and model_status=?";
                    List<AspModelPO> list = dbEngine.query(sql, AspModelPO.class, po.getModelType(), 0);
                    pageRouter = new ListPageRouter(list);
                    result = loadFromJson(po, pageRouter);
                }
            }

            if (result != null && pageRouter != null) {
                result.setFullName(pageRouter.loadFullName(key));
            }
        }

        return result;
    }

    private BpmPage loadFromJson(AspModelPO po, IPageRouter pageRouter) {
        BpmPage result = null;
        if (po != null) {
            UiPageLoader loader;

            if (po.getModelType() == AspModelPO.TYPE_UI_PC) {
                loader = new UiPcLoader(pageRouter);
            } else {
                loader = new UiMobileLoader(pageRouter);
            }

            result = loader.load(po.getModelContent());

            if (result == null) {
                log.error(loader.getLastError());
            } else {
                // 表存储或者json存储设置
                result.setDeploy(true);
                result.setId(po.getModelId());
//          result.getForm().setKey(po.getModelKey());
//          result.getForm().setTitle(po.getModelName());
//          result.setParentId(po.getModelParentId());
//          result.setUpdateAt(NumberTimeUtil.toTimestamp(po.getModelLastTime()));
            }
        }

        return result;
    }

//  public BpmPage get(Long pageId) {
//    return super.get(String.valueOf(pageId));
//  }

    public BpmDataset getDataset(String key) {
        BpmDataset result = null;
        int pos = key.lastIndexOf('.');

        if (pos > 0) {
            BpmPage bpmPage = loadFromOtherKey(key.substring(0, pos));
            if (bpmPage != null) {
                result = bpmPage.findDatasetByName(key.substring(pos + 1));
                if (result == null) {
                    log.error("not find dataset in page:" + key);
                }
            }
        }

        return result;
    }

    public BpmPage getByOtherKey(String otherKey) {
        BpmPage bean = getByKey(fn, otherKey);
        if (bean == null) {
            bean = loadFromOtherKey(otherKey);
            if (bean != null) {
                this.put(bean);
            }
        }
        return bean;
    }

    protected BpmPage loadFromOtherKey(String key) {
        BpmPage result = null;

        if (key != null) {
            if (bpmConfigBean.isDebug()) {
                // 部署界面数据
                String sql = "select model_id, model_site_id, model_full_name, model_name, model_status, model_content, model_type, model_version, model_use_vue, model_use_yaml, model_create_time, model_last_time from sw_bpm.bpm_model" +
                        " where model_full_name=? and model_type in (3,4)";
                AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, key);

                if (po != null) {
                    if (po.getModelUseYaml() != 0) {
                        result = loadFromYaml(key);
                    } else {
                        // TODO 路由缓存性能
                        sql = "select model_id, model_site_id, model_full_name, model_name, model_status, model_type from sw_bpm.bpm_model" +
                                " where model_type=? and model_status=?";
                        List<AspModelPO> list = dbEngine.query(sql, AspModelPO.class, po.getModelType(), 0);
                        result = loadFromJson(po, new ListPageRouter(list));
                    }
                }
            } else {
                result = loadFromYaml(key);
            }
        }

        if (result != null) {
            result.setFullName(key);
        }

        return result;
    }

    // 读yaml配置文件
    private BpmPage loadFromYaml(String key) {
        // 读取其他包的资源文件不能以 "/" 开头
        String path = "plugin/" + key.replace('.', '/') + ".yaml";
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        try (InputStream is = classLoader.getResourceAsStream(path)) {
            if (is != null) {
                UiPageLoader loader = new UiPcLoader(null);
                return loader.load(is);
            } else {
                log.error("not find resource: " + path);
            }
        } catch (IOException e) {
            log.error("load resource error: " + path, e);
        }

        return null;
    }
}
