package cc.smtweb.system.bpm.spring.dao;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.db.sqlbuilder.AbstractSelectSqlBuilder;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.builder.CardDataBuilder;
import cc.smtweb.system.bpm.core.ui.builder.SqlUtil;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldLookup;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class DatasetCardDao extends AbstractDatasetPlugin {
  public SwMap load(BpmPageContext ctx) {
    long id = ctx.getParamId();
    if (id != 0) {
      BpmDataset dataset = ctx.getDataset();
      // select
      AbstractSelectSqlBuilder sqlBuilder = makeSqlBuilderForTable(dataset, ctx.getBody(), false);
      sqlBuilder.addWhere(dataset.readIdFieldName(), id);

      return sqlBuilder.queryEntity(dbEngine, SwMap.class);
    }

    return null;
  }

  public Long save(BpmPageContext ctx) {
    CardDataBuilder builder = CardDataBuilder.create(dbEngine, ctx.getDataset(), ctx.getBody(), ctx.getSession());
    return builder.update();
  }

  public int remove(BpmPageContext ctx) {
    String ids = ctx.getBody().readString(IParamConst.PARAMS_IDS);

    BpmDataset dataset = ctx.getDataset();
    if (StringUtils.isNotBlank(ids)) {
      return dbEngine.update("delete from " + dataset.fullName() + " where " + dataset.readIdFieldName() + " in(" + ids + ")");
    } else {
      long id = ctx.getParamId();
      if (id != 0) {
        return dbEngine.update("delete from " + dataset.fullName() + " where " + dataset.readIdFieldName() + "=?", id);
      }
    }

    return 0;
  }

  public BpmKeyValue loadLookup(BpmPageContext ctx, Object lookupKey) {
    BpmFieldLookup fieldLookup = ctx.getFieldLookup();
    if (fieldLookup.isManyToMany()) {
      StringBuilder sb = makeLookupSql(fieldLookup, true).append("=?");
      List<BpmKeyManyValue> list = dbEngine.query(sb.toString(), BpmKeyManyValue.class, lookupKey);
      BpmKeyValue result = new BpmKeyValue();

      result.setOptKey(SqlUtil.join(list, BpmKeyManyValue::getOptChildKey));
      result.setOptValue(SqlUtil.join(list, BpmKeyManyValue::getOptValue));

      return result;
    } else {
      StringBuilder sb = makeLookupSql(fieldLookup, false).append("=?");
      return dbEngine.queryEntity(sb.toString(), BpmKeyValue.class, lookupKey);
    }
  }

}
