package cc.smtweb.system.bpm.spring.dao;

import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.exception.BpmException;
import cc.smtweb.system.bpm.core.exception.BpmIlegalArgumentException;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.engine.entity.AspModelPO;
import cc.smtweb.system.bpm.spring.cache.BpmPageCache;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class DatasetConfigDao {
    @SwParam
    private BpmPageCache bpmPageCache;

    @Autowired
    private DbEngine dbEngine;

    public BpmPage loadPage(Long pageId) {
        if (pageId == null) {
            throw new BpmException("参数 pageId 必填");
        }

        BpmPage uiPage = bpmPageCache.get(pageId);

        if (uiPage == null) {
            throw new BpmException("未找到页面, id=" + pageId);
        }

        return uiPage;
    }

    public BpmPage loadPage(String pageName) {
        if (pageName == null) {
            throw new BpmException("参数 pageName 必填");
        }

        BpmPage uiPage = bpmPageCache.get(pageName);
        if (uiPage == null) {
            uiPage = bpmPageCache.getByOtherKey(pageName);
        }

        if (uiPage == null) {
            throw new BpmException("未找到页面, name=" + pageName);
        }

        return uiPage;
    }

//  private BpmDataset findUiDataset(BpmPage uiPage, String datasetName) {
//    if (datasetName == null) {
//      throw new BpmException("参数 dataset 必填");
//    }
//
//    BpmDataset dataset = uiPage.findDatasetByName(datasetName);
//
//    if (dataset == null) {
//      throw new BpmException("未找到dataset, pageId=" + uiPage.getId() + ", id=" + datasetName);
//    }
//
//    return dataset;
//  }

//  public void loadDataset(BpmPageContext pageContext) {
//    SwMap body = pageContext.getBody();
//    String pageName = body.readString(IParamConst.PARAM_PAGE_NAME);
//    if (pageName == null) {
//      pageName = body.readString(IParamConst.PARAM_PAGE_ID);
//    }
//
//    BpmPage page = loadPage(pageName);
//    pageContext.setDeploy(page.isDeploy());
//    pageContext.setDataset(findUiDataset(page, body.readString(IParamConst.PARAM_DATASET)));
//  }

    public void loadDataset(BpmPageContext pageContext, String datasetPath) {
        if (datasetPath != null) {
            BpmDataset pageDataset = bpmPageCache.getDataset(datasetPath);

            if (pageDataset == null) {
                throw new BpmIlegalArgumentException("not find dataset: " + datasetPath);
            }

            pageContext.setDeploy(true);
            pageContext.setDataset(pageDataset);
//    } else {
//      loadDataset(pageContext);
        }
    }

    /**
     * 读取页面全路径
     *
     * @param uiPage        页面对象
     * @param includeModule 是否包含模块路径
     * @return 页面全路径，全局唯一
     */
    public List<String> loadPagePath(BpmPage uiPage, boolean includeModule) {
        String sql = "select model_id, model_parent_id, model_key, model_name from sw_bpm.asp_model where model_id=?";
        AspModelPO curPage = dbEngine.queryEntity(sql, AspModelPO.class, uiPage.getId());

        if (StringUtils.isBlank(curPage.getModelKey())) {
            throw new BpmException("页面 '" + curPage.getModelName() + "' 未设置名称");
        }

        List<String> names = new ArrayList<>();

        Long parentId = curPage.getModelParentId();
        while (parentId != null) {
            AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, parentId);

            if (StringUtils.isBlank(po.getModelKey())) {
                throw new BpmException("页面 '" + po.getModelName() + "' 未设置名称");
            }

            names.add(0, po.getModelKey());
            parentId = po.getModelParentId();
        }

        if (includeModule) {
            String module = dbEngine.queryString("SELECT mc_module FROM sw_bpm.asp_model_catalog WHERE mc_id IN (SELECT model_mc_id FROM sw_bpm.asp_model WHERE model_id=?)", uiPage.getId());
            if (StringUtils.isBlank(module)) {
                throw new BpmException("未设置服务模块名称");
            }

            names.add(0, module);
        }

        names.add(curPage.getModelKey());

        return names;
    }
}
