package cc.smtweb.system.bpm.spring.dao;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.db.sqlbuilder.AbstractSelectSqlBuilder;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmMapData;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.builder.SqlUtil;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldFilter;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldLookup;
import cc.smtweb.system.bpm.core.ui.entity.dataset.IBpmDatasetConst;
import cc.smtweb.system.bpm.engine.ui.entity.consts.MappingTypeEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
public class DatasetListDao extends AbstractDatasetPlugin {
  public List<SwMap> list(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    SwMap query = ctx.getBody();
    // select
    AbstractSelectSqlBuilder sqlBuilder = makeSqlBuilderForTable(dataset, query, false);

    int limit = query.readInt(IParamConst.PARAM_PAGE_SIZE, IParamConst.PAGE_DEFAULT_SIZE);
    int start = query.readInt(IParamConst.PARAM_PAGE_INDEX, 0) * limit;

    return sqlBuilder.pagedQuery(dbEngine, SwMap.class, start, limit);
  }

  public SwMap count(BpmPageContext ctx) {
    AbstractSelectSqlBuilder sqlBuilder = makeSqlBuilderForTable(ctx.getDataset(), ctx.getBody(), true);

    // select
    return sqlBuilder.queryEntity(dbEngine, SwMap.class);
  }

  public BpmMapData listLookup(BpmPageContext ctx, Set<String> idSet) {
    BpmFieldLookup fieldLookup = ctx.getFieldLookup();
    if (fieldLookup.getMappingType().equals(MappingTypeEnum.manyToMany)) {
      return listLookupManyToMany(fieldLookup, idSet);
    } else {
      return listLookupManyToOne(fieldLookup, idSet);
    }
  }

  public List<SwMap> listAll(BpmPageContext ctx) {
    // select
    AbstractSelectSqlBuilder sqlBuilder = makeSqlBuilderForTable(ctx.getDataset(), ctx.getBody(), ctx.getFieldLookup().getFilters());

    return sqlBuilder.query(dbEngine, SwMap.class);
  }

  private AbstractSelectSqlBuilder makeSqlBuilderForTable(BpmDataset dataset, SwMap query, List<BpmFieldFilter> fieldFilters) {
    AbstractSelectSqlBuilder sqlBuilder = createSqlBuilderForTable(dataset, false);

    List<Map<String, Object>> filters = (List<Map<String, Object>>)query.get(SqlUtil.PN_FILTER);

    if (fieldFilters != null) {
      for (BpmFieldFilter fieldFilter : fieldFilters) {
        String keyword = fieldFilter.getKeyword();
        String fieldName = fieldFilter.getField();

        if (StringUtils.isNotBlank(keyword) && StringUtils.isNotBlank(fieldName)) {
          // 级联动态数据
          if (fieldFilter.isCascade()) {
            if (filters != null) {
              for (Map<String, Object> filter : filters) {
                String queryField = (String) filter.get(IBpmDatasetConst.PN_ITEM_FIELD);

                if (fieldName.equals(queryField)) {
                  String queryKeyword = (String) filter.get(IBpmDatasetConst.PN_ITEM_KEYWORD);
                  if (StringUtils.isNotBlank(queryKeyword)) {
                    SqlUtil.addSqlWhere(sqlBuilder, fieldName, fieldFilter.getOp(), queryKeyword);
                  }
                  break;
                }
              }
            }
          } else {
            SqlUtil.addSqlWhere(sqlBuilder, fieldName, fieldFilter.getOp(), keyword);
          }
        }
      }
    }

    return sqlBuilder;
  }

  private BpmMapData listLookupManyToOne(BpmFieldLookup fieldLookup, Set<String> ids) {
    StringBuilder sb = makeLookupSql(fieldLookup, false)
            .append(" IN (").append(String.join(",", ids)).append(")");

    List<BpmKeyValue> list = dbEngine.query(sb.toString(), BpmKeyValue.class);

    BpmMapData result = new BpmMapData(list.size());
    for (BpmKeyValue item : list) {
      result.put(item.getOptKey(), item.getOptValue());
    }

    return result;
  }

  private BpmMapData listLookupManyToMany(BpmFieldLookup fieldLookup, Set<String> ids) {
    // id, 子表ID, 子表值
    // select m.key, b.key, b.value from m left join b on(b.key = m.value) where m.key in ...
    StringBuilder sb = makeLookupSql(fieldLookup, true)
            .append(" IN (").append(String.join(",", ids)).append(")");

    List<BpmKeyManyValue> list = dbEngine.query(sb.toString(), BpmKeyManyValue.class);

    BpmMapData result = new BpmMapData(list.size());
    for (BpmKeyManyValue item : list) {
      String value = item.getOptValue();

      if (StringUtils.isNotBlank(value)) {
        result.merge(item.getOptKey(), value, (a, b) -> a + "," + b);
      }
    }

    return result;
  }
}
