package cc.smtweb.system.bpm.web.design.codegen;

import cc.smtweb.system.bpm.util.CodeGenUtil;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class JavaPath {
  private final String rootPath;
  private final String[] rootPackages;
  private List<String> packageNames;
  @Getter
  private String className;
  @Getter
  private String moduleName;

  public JavaPath(String rootPath, String rootPackage) {
    this.rootPath = rootPath;
    this.rootPackages = rootPackage.split("\\.");
  }

  public void setPackages(String[] names) {
    this.packageNames = null;
    this.moduleName = names[0];
    for (int i = 1; i < names.length; i++) {
      addPackage(names[i]);
    }
  }

  public JavaPath addPackage(String name) {
    if (packageNames == null) {
      packageNames = new ArrayList<>();
    }

    packageNames.add(name.toLowerCase());
    return this;
  }

  public boolean makeDirs() {
    File servicePath = new File(getAbsolutePath());

    if (servicePath.exists()) {
      return true;
    }

    return servicePath.mkdirs();
  }

  public String getAbsolutePath() {
    return rootPath + String.join("/", rootPackages) + "/" + StringUtils.join(packageNames, "/");
  }

  public JavaPath setPackageUrl(String customApi) {
    return setPackageUrl(customApi.split("/"));
  }

  public JavaPath setPackageUrl(String[] names) {
    this.packageNames = null;
    for (int i = 0; i < names.length - 1; i++) {
      addPackage(names[i].toLowerCase());
    }

    this.className = CodeGenUtil.toUpperHump(names[names.length - 1]);
    return this;
  }

  public void setClassName(String className) {
    this.className = CodeGenUtil.toUpperHump(className);
  }

  public String getPackage() {
    return StringUtils.join(rootPackages, ".") + "." + StringUtils.join(packageNames, ".");
  }
}
