package cc.smtweb.system.bpm.web.design.codegen;

import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.form.BpmForm;
import cc.smtweb.system.bpm.core.ui.entity.form.BpmWidget;
import cc.smtweb.system.bpm.util.CodeGenUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class UiPageRefLoader {
  // 处理button link 页面
  protected UiPageRef loadPageRef(BpmPage bpmPage, String dir, String routerName) {
    UiPageRef pageRef = new UiPageRef();
    pageRef.setId(bpmPage.getId());
    pageRef.setName(bpmPage.getFullName());
    pageRef.setDir(dir);
    pageRef.setRouterName(routerName);
    for (BpmForm form: bpmPage.getForms()) {
      buildWidgets(form, pageRef);
    }

    return pageRef;
  }

  private void buildWidgets(BpmWidget parent, UiPageRef pageRef) {
    // 生成响应对象
    if (parent.getVars() != null) {
      Map<String, Object> props = new HashMap<>(parent.getVars().size());
      for (Map.Entry<String, String> entry: parent.getVars().entrySet()) {
         String key = entry.getKey();
         Object value = parent.getProps().get(key);
         props.put(key, value);
      }

      if (!props.isEmpty()) {
        pageRef.addWidget(parent.getName(), props);
      }
    }

    // 循环子控件
    if (parent.getChildren() != null) {
      for (BpmWidget widget: parent.getChildren()) {
        buildWidgets(widget, pageRef);
      }
    }
  }

  String getRouterName(List<String> names) {
    StringBuilder sb = new StringBuilder();
    for (String name: names) {
      sb.append(CodeGenUtil.toUpperHump(name));
    }

    return sb.toString();
  }
}
