package cc.smtweb.system.bpm.web.design.db;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.cache.ModelDatabaseCache;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelDatabase;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzDatasetPropsVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzDatasetVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.dataset.FzFieldVO;
import cc.smtweb.system.bpm.engine.ui.entity.vo.widiget.UiControlPropsVO;
import cc.smtweb.system.bpm.util.CodeGenUtil;
import org.apache.commons.lang3.StringUtils;

import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.util.*;

@SwService
public class DbQueryDatasetService {
  @SwParam
  private DbEngine dbEngine;

  public R query(@SwBody SwMap body) {
    String sql = body.readString("sql");

    if (StringUtils.isBlank(sql)) {
      return R.error("sql参数为空");
    }

    if(!sql.trim().toLowerCase().startsWith("select ")){
      return R.error("sql必须是查询语句");
    }

    if (sql.endsWith(";")) {
      return R.error("sql语句需要去掉结束符合';'");
    }

    ModelDatabase modelDatabase = getModelDatabase(sql);

    FzDatasetVO uiDataset = new FzDatasetVO();
    uiDataset.setType("fz-dataset");
    uiDataset.setProps(new FzDatasetPropsVO());

    dbEngine.query(sql + " limit 1", resultSet -> {
      // 解析数据库中的库名
      ModelTable modelTable = null;
      ResultSetMetaData rsmd = resultSet.getMetaData();
      int columnCount = rsmd.getColumnCount();
      List<FzFieldVO> fields = new ArrayList<>();
      String masterTableName = null;
      String masterTableLabel = null;

      for (int index = 1; index <= columnCount; ++index) {
        FzFieldVO uiField = new FzFieldVO();
        UiControlPropsVO props = new UiControlPropsVO();
        uiField.setProps(props);

        String fieldName = rsmd.getColumnName(index);
        String name = rsmd.getColumnLabel(index);
        int size = rsmd.getColumnDisplaySize(index);
        uiField.setId(name);
        props.put("fieldName", fieldName);
        props.put("name", CodeGenUtil.underlineToHump(name));
        props.put("length", size);

        switch (rsmd.getColumnType(index)) {
          case Types.BIGINT:
            uiField.setType("fz-field-long");
            props.put("dataType", "id");
            break;
          case Types.INTEGER:
          case Types.SMALLINT:
          case Types.TINYINT:
            uiField.setType("fz-field-int");
            props.put("dataType", "int");
            break;
          case Types.CHAR:
          case Types.NCHAR:
          case Types.VARCHAR:
          case Types.NVARCHAR:
            uiField.setType("fz-field-string");
            props.put("dataType", "string");
            break;
          case Types.FLOAT:
            uiField.setType("fz-field-float");
            props.put("dataType", "float");
            break;
          case Types.DATE:
          case Types.TIME:
          case Types.TIMESTAMP:
            uiField.setType("fz-field-date");
            props.put("dataType", "date");
            break;
          default:
            throw new IllegalStateException("Unexpected value: " + rsmd.getColumnType(index));
        }

        String tableName = rsmd.getTableName(index);

        if (modelDatabase != null) {
          modelTable = findModelTable(modelDatabase, modelTable, tableName);

          if (modelTable != null) {
            ModelField modelField = modelTable.findFieldByName(name);

            if (modelField != null) {
              props.put("label", modelField.getTitle());
            }
            if (masterTableName == null) {
              masterTableName = tableName;
              masterTableLabel = modelTable.getTitle();
            }
          }
        }

        if (masterTableName == null) {
          masterTableName = tableName;
          masterTableLabel = tableName;
        }

        fields.add(uiField);
      }

      uiDataset.getProps().setFields(fields.toArray(new FzFieldVO[0]));
      uiDataset.getProps().setName(masterTableName);
      uiDataset.getProps().setLabel(masterTableLabel);

      return null;
    });

    return R.success(uiDataset);
  }

  private ModelDatabase getModelDatabase(String sql) {
    ModelDatabase modelDatabase = null;
    String dbName = extractSqlDbName(sql);
    Long modelId = dbEngine.queryLong("select model_id from sw_bpm.asp_model where model_key=? and model_type=1", dbName);
    if (modelId != null) {
      modelDatabase = ModelDatabaseCache.getInstance().get(modelId);
    }

    return modelDatabase;
  }

  private ModelTable findModelTable(ModelDatabase modelDatabase, ModelTable modelTable, String tableName) {
    if (tableName != null) {
      if (modelTable != null && tableName.equalsIgnoreCase(modelTable.getName())) {
        return modelTable;
      }

      return  ModelTableCache.getInstance().getByName(tableName);
    }

    return null;
  }

  private static final Set<String> KEYWORD_SET = new HashSet<String>(){{
    this.add("from");
    this.add("join");
  }};

  public static String extractSqlDbName(String sql) {
    StringTokenizer st = new StringTokenizer(sql, " ");

    int step = 0;
    while (st.hasMoreTokens()) {
      String token = st.nextToken();
      if (step == 0) {
        if (KEYWORD_SET.contains(token.toLowerCase())) {
          step = 1;
        }
      } else {
        String[] ary = token.split("\\.");
        if (ary.length > 1) {
          return ary[0];
        }

        step = 0;
      }
    }

    return null;
  }
}
