package cc.smtweb.system.bpm.web.design.model;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.cache.ModelDatabaseCache;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.config.DesignConfig;
import cc.smtweb.framework.core.db.sqlbuilder.InsertSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.UpdateSqlBuilder;
import cc.smtweb.framework.core.db.vo.ModelDatabase;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;
import org.apache.commons.lang3.StringUtils;

//@Api(tags = "数据库设计")
@SwService
public class AspDbCardService {
    @SwParam
    private DbEngine dbEngine;

    @SwParam
    private ModelDatabaseCache bpmDbCache;
    private ModelTableCache bpmTableCache;

    @SwParam
    private DesignConfig designConfig;

    @SwParam
    protected DatasetConfigDao datasetConfigDao;

    //  @ApiOperation(value = "读取数据模型数据")
    public R load(@SwParam("id") long id, UserSession us) {
        ModelDatabase db = bpmDbCache.get(id);
        if (db == null) {
            return R.error("没有找到指定数据库定义（" + id + "）！");
        }

        return R.success(db);
    }

    public R loadByKey(@SwParam("key") String key, @SwParam("type") String type, UserSession us) {
        ModelDatabase db = bpmDbCache.getByName(key);
        if (db == null) {
            return R.error("没有找到指定数据库定义（" + key + "）！");
        }
        return R.success(db);
    }

    private boolean isNullId(Long id) {
        return id == null || id == 0;
    }

    //  @ApiOperation(value = "创建数据模型")
    public R save(@SwBody ModelDatabase modelPo, UserSession us) {

        if (StringUtils.isBlank(modelPo.getName())) {
            return R.error("库名不能为空");
        }
        if (StringUtils.isBlank(modelPo.getTitle())) {
            return R.error("库标题不能为空");
        }

        modelPo.setUpdateAt(DateUtil.nowDateTimeLong());

        // 保存数据到部署表和部署历史表
        if (isNullId(modelPo.getId())) {
            dbEngine.doTrans(() -> {
                modelPo.setId(dbEngine.nextId());

                InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.asp_model_database");
                sqlBuilder.add("db_id", modelPo.getId())
                        .add("db_name", modelPo.getName())
                        .add("db_title", modelPo.getTitle())
                        .add("db_status", modelPo.getStatus())
                        .add("db_version", modelPo.getVersion())
                        .add("db_create_at", modelPo.getCreateAt())
                        .add("db_update_at", modelPo.getUpdateAt())
                        .add("db_create_uid", us.getUserId())
                        .add("db_update_uid", us.getUserId());
                sqlBuilder.update(dbEngine);

                return true;
            });

            bpmDbCache.put(modelPo);

            return R.success(modelPo.getId());
        } else {
            modelPo.setVersion(modelPo.getVersion() + 1);

            // 更新数据
            boolean r = dbEngine.doTrans(() -> {

                UpdateSqlBuilder sqlBuilder = SqlBuilder.createUpdate("sw_bpm.asp_model_database");

                sqlBuilder.add("db_name", modelPo.getName())
                        .add("db_title", modelPo.getTitle())
                        .add("db_status", modelPo.getStatus())
                        .add("db_version", modelPo.getVersion())
                        .add("db_update_at", modelPo.getUpdateAt())
                        .add("db_update_uid", us.getUserId());

                int ret = sqlBuilder.update(dbEngine);

                if (ret > 0) {
                    bpmDbCache.put(modelPo);
                    return true;
                } else {
                    bpmDbCache.reset(modelPo);
                }

                return false;
            });


            return R.error("保存失败，未找到记录");
        }
    }

    //  @ApiOperation(value = "删除数据模型")
    public R delete(@SwBody ModelDatabase modelPo, UserSession us) {
        String sql = "delete from sw_bpm.asp_model_database where model_id=?";
        if (dbEngine.update(sql, modelPo.getId()) > 0) {
            bpmDbCache.remove(modelPo.getId());
            return R.success();
        }

        return R.error("删除失败");
    }
}
