package cc.smtweb.system.bpm.web.design.model;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.config.DesignConfig;
import cc.smtweb.framework.core.db.sqlbuilder.InsertSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.UpdateSqlBuilder;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.engine.AbstractLoader;
import cc.smtweb.system.bpm.engine.entity.AspModelPO;
import cc.smtweb.system.bpm.engine.entity.AspModelTreeVO;
import cc.smtweb.system.bpm.engine.process.loader.FlowLoader;
import cc.smtweb.system.bpm.engine.ui.loader.IPageRouter;
import cc.smtweb.system.bpm.engine.ui.loader.TreePageRouter;
import cc.smtweb.system.bpm.engine.ui.loader.UiMobileLoader;
import cc.smtweb.system.bpm.engine.ui.loader.UiPcLoader;
import cc.smtweb.system.bpm.spring.cache.BpmFlowCache;
import cc.smtweb.system.bpm.spring.cache.BpmPageCache;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;
import cc.smtweb.system.bpm.util.ITreeDataLevelHandler;
import cc.smtweb.system.bpm.util.TreeDataUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

//@Api(tags = "数据模型")
@SwService
public class AspModelCardService {
  @SwParam
  private DbEngine dbEngine;

  @SwParam
  private BpmFlowCache bpmFlowCache;

  @SwParam
  private BpmPageCache bpmPageCache;

  @SwParam
  private DesignConfig designConfig;

  @SwParam
  protected DatasetConfigDao datasetConfigDao;

  //  @ApiOperation(value = "读取数据模型数据")
  public R load(@SwParam("id") long id, UserSession us) {
    String sql = "select model_id, model_parent_id, model_mc_id, model_site_id, model_key, model_name, model_status, model_content, model_type, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
            " where model_id=? and model_site_id=?";
    AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, id, us.getSiteId());
    return R.success(po);
  }

  public R loadByKey(@SwParam("key") String key, @SwParam("type") String type, UserSession us) {
    String sql = "select model_id, model_parent_id, model_mc_id, model_site_id, model_key, model_name, model_status, model_content, model_type, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
            " where model_key=? and model_type=? and model_site_id=?";
    AspModelPO po = dbEngine.queryEntity(sql, AspModelPO.class, key, type, us.getSiteId());
    return R.success(po);
  }

  private boolean isNullId(Long id) {
    return id == null || id == 0;
  }

//  @ApiOperation(value = "创建数据模型")
  public R save(@SwParam("commit") boolean commit, @SwBody AspModelPO modelPo, UserSession us) {
    String modelContent = modelPo.getModelContent();
    if (StringUtils.isBlank(modelContent)) {
      return R.error("内容不能为空");
    }

    if (commit) {
      // 检查数据
      AbstractLoader loader = null;
      switch (modelPo.getModelType()) {
        case AspModelPO.TYPE_FLOW:
          loader = new FlowLoader();
          break;
        case AspModelPO.TYPE_UI_PC:
          loader = new UiPcLoader(loadTree(modelPo.getModelId()));
          break;
        case AspModelPO.TYPE_UI_MOBILE:
          loader = new UiMobileLoader(loadTree(modelPo.getModelId()));
          break;
        default:
          return R.error("error model type: " + modelPo.getModelType());
      }

      Object page = loader.load(modelContent);

      if (loader.getLastError() != null) {
        return R.error(loader.getLastError()).put("modelErrors", loader.getErrors());
      }

      // 检测 modern + path
      if (page instanceof BpmPage) {
        // 检测和读取页面目录
        BpmPage uiPage = (BpmPage)page;
        uiPage.setId(modelPo.getModelId());
//        uiPage.getForm().setKey(modelPo.getModelKey());
//        uiPage.getForm().setTitle(modelPo.getModelName());
//        uiPage.setParentId(modelPo.getModelParentId());

        List<String> names = datasetConfigDao.loadPagePath(uiPage, true);
        modelPo.setModelFullName(String.join(".", names));
      }

      modelPo.setModelStatus(0);
      // 计算签名
      modelPo.setModelContentSign(DigestUtils.sha256Hex(modelContent) + "," + modelContent.length());
    } else {
      modelPo.setModelStatus(-1);
    }

    modelPo.setModelLastTime(DateUtil.nowDateTimeLong());

    // 保存数据到部署表和部署历史表
    if (isNullId(modelPo.getModelId())) {
      if (isNullId(modelPo.getModelMcId()) && isUiType(modelPo)) {
        return R.error("页面缺少分类ID");
      }

      dbEngine.doTrans(() -> {
        modelPo.setModelId(dbEngine.nextId());
        modelPo.setModelVersion(commit ? 1 : 0);

        InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.asp_model");
        sqlBuilder.add("model_id", modelPo.getModelId())
                .add("model_parent_id", modelPo.getModelParentId())
                .add("model_mc_id", modelPo.getModelMcId())
                .add("model_site_id", us.getSiteId())
                .add("model_key", modelPo.getModelKey())
                .add("model_name", modelPo.getModelName())
                .add("model_status", modelPo.getModelStatus())
                .add("model_content", modelContent)
//                .add("model_content_sign", contentSign)
                .add("model_type", modelPo.getModelType())
                .add("model_sub_type", modelPo.getModelSubType())
                .add("model_version", modelPo.getModelVersion())
                .add("model_create_time", modelPo.getModelLastTime())
                .add("model_last_time", modelPo.getModelLastTime())
                .add("model_create_uid", us.getUserId())
                .add("model_update_uid", us.getUserId());
        sqlBuilder.update(dbEngine);

        if (commit) {
          this.insertDeployLog(modelPo, us);
        }

        return true;
      });

      updateCache(modelPo.getModelId(), modelPo.getModelType());

      return R.success(modelPo.getModelId());
    } else {
      if (commit) {
        // 比较签名
        AspModelPO old = dbEngine.queryEntity("select model_version, model_content_sign from sw_bpm.bpm_model where model_id=?", AspModelPO.class, modelPo.getModelId());
        if (old != null) {
          if (old.getModelContentSign() != null && old.getModelContentSign().equals(modelPo.getModelContentSign())) {
            return R.success();
          }

          modelPo.setModelVersion(old.getModelVersion() + 1);
        } else {
          modelPo.setModelVersion(1);
        }
      }

      // 更新数据
      boolean r = dbEngine.doTrans(() -> {
//        Timestamp now = new Timestamp(System.currentTimeMillis());
        UpdateSqlBuilder sqlBuilder = SqlBuilder.createUpdate("sw_bpm.asp_model");
//                .add("model_parent_id", modelPo.getModelParentId())
//                .add("model_mc_id", modelPo.getModelMcId())
        sqlBuilder.add("model_key", modelPo.getModelKey())
                .add("model_name", modelPo.getModelName())
                .add("model_content", modelContent)
//                .add("model_content_sign", contentSign)
                .add("model_status", modelPo.getModelStatus())
                .add("model_update_uid", us.getUserId())
                .add("model_last_time", modelPo.getModelLastTime())
                .addWhere("model_id", modelPo.getModelId())
                .addWhere("model_site_id", us.getSiteId());

        if (commit) {
          sqlBuilder.add("model_version", modelPo.getModelVersion());
        }

        int ret = sqlBuilder.update(dbEngine);

        if (ret > 0) {
          if (commit) {
            this.insertDeployLog(modelPo, us);
          }

          return true;
        }

        return false;
      });

      if (r) {
        updateCache(modelPo.getModelId(), modelPo.getModelType());
        return R.success();
      }

      return R.error("保存失败，未找到记录");
    }
  }

  private IPageRouter loadTree(Long pageId) {
    ITreeDataLevelHandler<AspModelTreeVO> handler = AspModelTreeVO.createTreeHandler();

    List<AspModelTreeVO> list = dbEngine.query("select model_id, model_parent_id, model_key, model_name from sw_bpm.asp_model where model_mc_id in" +
                    " (select model_mc_id from sw_bpm.asp_model where model_id=?)",
            AspModelTreeVO.class, pageId);
    AspModelTreeVO root = new AspModelTreeVO();
    TreeDataUtil.buildLevelTree(root, list, handler);
    return new TreePageRouter(root, null);
  }

  private boolean isUiType(@SwBody AspModelPO modelPo) {
    return modelPo.getModelType() == AspModelPO.TYPE_UI_PC || modelPo.getModelType() == AspModelPO.TYPE_UI_MOBILE;
  }

  private void insertDeployLog(AspModelPO modelPo, UserSession us) {
    // 部署
    Long oldId = dbEngine.queryLong("select model_id from sw_bpm.bpm_model where model_id=?", modelPo.getModelId());

    if (oldId != null) {
      UpdateSqlBuilder sqlBuilder = SqlBuilder.createUpdate("sw_bpm.bpm_model");
      sqlBuilder.add("model_full_name", modelPo.getModelFullName())
              .add("model_name", modelPo.getModelName())
              .add("model_status", 0)
              .add("model_use_vue", modelPo.getModelUseVue())
              .add("model_use_yaml", modelPo.getModelUseYaml())
              .add("model_content", modelPo.getModelContent())
              .add("model_content_sign", modelPo.getModelContentSign())
              .add("model_version", modelPo.getModelVersion())
              .add("model_last_time", modelPo.getModelLastTime())
              .addWhere("model_id", modelPo.getModelId());
      sqlBuilder.update(dbEngine);
    } else {
      InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.bpm_model");
      sqlBuilder.add("model_id", modelPo.getModelId())
              .add("model_full_name", modelPo.getModelFullName())
              .add("model_site_id", us.getSiteId())
              .add("model_name", modelPo.getModelName())
              .add("model_status", 0)
              .add("model_content", modelPo.getModelContent())
              .add("model_content_sign", modelPo.getModelContentSign())
              .add("model_type", modelPo.getModelType())
              .add("model_version", modelPo.getModelVersion())
              .add("model_create_time", modelPo.getModelLastTime())
              .add("model_last_time", modelPo.getModelLastTime());
      sqlBuilder.update(dbEngine);
    }

    // 产生部署日志
    Long dmId = dbEngine.nextId();
    InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.asp_model_deploy");
    sqlBuilder.add("md_id", dmId)
            .add("md_model_id", modelPo.getModelId())
            .add("md_model_key", modelPo.getModelKey())
            .add("md_name", modelPo.getModelName())
            .add("md_status", modelPo.getModelStatus())
            .add("md_content", modelPo.getModelContent())
            .add("md_type", modelPo.getModelType())
            .add("md_version", modelPo.getModelVersion())
            .add("md_create_uid", us.getUserId());
    sqlBuilder.update(dbEngine);
  }

  private void updateCache(Long modelId, byte modelType) {
    switch (modelType) {
      case AspModelPO.TYPE_FLOW:
        bpmFlowCache.remove(modelId);
        break;
      case AspModelPO.TYPE_UI_PC:
      case AspModelPO.TYPE_UI_MOBILE:
        bpmPageCache.remove(modelId);
        break;
      default:
    }
  }

  //  @ApiOperation(value = "删除数据模型")
  public R delete(@SwBody AspModelPO modelPo, UserSession us) {
    String sql = "delete from sw_bpm.asp_model where model_id=? and model_site_id=?";
    if (dbEngine.update(sql, modelPo.getModelId(), us.getSiteId()) > 0) {
      updateCache(modelPo.getModelId(), modelPo.getModelType());
      return R.success();
    }

    return R.error("删除失败");
  }

  // 修改子类型
  public R saveSubType(@SwBody AspModelPO modelPo, UserSession us) {
    dbEngine.update("update sw_bpm.asp_model set model_sub_type=? where model_id=? and model_site_id=?", modelPo.getModelSubType(), modelPo.getModelId(), us.getSiteId());
    return R.success();
  }
}
