package cc.smtweb.system.bpm.web.design.model;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.config.DesignConfig;
import cc.smtweb.system.bpm.spring.BpmConfigBean;
import cc.smtweb.system.bpm.util.YamlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@SwService
public class AspModelConfigService {
  @SwParam
  private DesignConfig designConfig;

  @SwParam
  private BpmConfigBean bpmConfigBean;

  public R load() {
    R result = R.success().put("dataType", designConfig.getDataTypes());
    String configUiPath = bpmConfigBean.getConfigUiPath();

    if (StringUtils.isNotBlank(configUiPath)) {
      File file = new File(configUiPath);
      if (file.isDirectory()) {
        File[] files = file.listFiles();
        if (files != null) {
          Map<String, SwMap> config = new HashMap<>();
          for (File yamlFile: files) {
            String fileName = yamlFile.getName();
            if (fileName.endsWith(".yaml") && fileName.startsWith("fx-")) {
              SwMap meta = YamlUtil.readValue(yamlFile, SwMap.class);
              config.put(fileName.substring(0, fileName.length() - 5), meta);
            }
          }

          if (!config.isEmpty()) {
            result.put("schema", config);
          }
        }
      } else {
        log.error("not find path:" + configUiPath);
      }
    }

    return result;
  }
}
