package cc.smtweb.system.bpm.web.design.model;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.system.bpm.engine.entity.AspModelCatalogPO;
import cc.smtweb.system.bpm.engine.entity.AspModelPO;
import cc.smtweb.system.bpm.engine.entity.AspModelTreeVO;
import cc.smtweb.system.bpm.util.TreeDataUtil;
import cc.smtweb.system.bpm.web.design.model.entity.TreeMoveVO;

import java.util.ArrayList;
import java.util.List;

@SwService
public class AspModelTreeService {
  @SwParam
  private DbEngine dbEngine;

  public R treeAll(@SwParam("mcId") long mcId, @SwParam("modelType") long modelType, UserSession us) {
    return R.success(loadTreeAll(mcId, modelType, us));
  }

  private List<AspModelTreeVO> loadTreeAll(@SwParam("mcId") long mcId, @SwParam("modelType") long modelType, UserSession us) {
    String sql = "select model_id, model_parent_id, model_key, model_name, model_status, model_type, model_sub_type, model_version, model_create_time, model_last_time from sw_bpm.asp_model" +
            " where model_mc_id=? and model_type=? and model_site_id=? order by model_order, model_id";
    List<AspModelTreeVO> list = dbEngine.query(sql, AspModelTreeVO.class, mcId, modelType, us.getSiteId());

    AspModelTreeVO root = new AspModelTreeVO();
    root.setModelName("项目");

    return TreeDataUtil.buildTree(root, list, AspModelTreeVO.createTreeHandler());
  }

  public R menu(@SwParam("mcId") long mcId, @SwParam("modelType") long modelType, UserSession us) {
    List<AspModelTreeVO> tree = loadTreeAll(mcId, modelType, us);

    // 强制只显示2级
    if (tree != null) {
      for (AspModelTreeVO item : tree) {
        List<AspModelTreeVO> children = item.getChildren();

        if (children != null) {
          for (AspModelTreeVO child : item.getChildren()) {
            child.setChildren(null);
          }
        }
      }
    }

    return R.success(tree);
  }

  /** 动态Tree数据 */
  public R tree(@SwParam("type") long type, @SwParam("catalog") long catalog, @SwParam("parentId") Long parentId, UserSession us) {
    R result = null;
    if (parentId == null) {
      String sql = "select model_id, model_mc_id, model_key, model_name, model_status, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
              " where model_mc_id=? and model_parent_id is null and model_site_id=? and model_type=? order by model_order, model_id";
      List<AspModelPO> list = dbEngine.query(sql, AspModelPO.class, catalog, us.getSiteId(), type);
      result = R.success(list);

      sql = "select mc_id, mc_parent_id, mc_code, mc_site_id, mc_name, mc_create_time, mc_last_time from sw_bpm.asp_model_catalog" +
              " where mc_id=? and mc_site_id=?";
      AspModelCatalogPO po = dbEngine.queryEntity(sql, AspModelCatalogPO.class, catalog, us.getSiteId());
      result.put("catalog", po);
    } else {
      String sql = "select model_id, model_mc_id, model_key, model_name, model_status, model_version, model_create_uid, model_update_uid, model_create_time, model_last_time from sw_bpm.asp_model" +
              " where model_mc_id=? and model_parent_id=? and model_site_id=? and model_type=? order by model_order, model_id";
      List<AspModelPO> list = dbEngine.query(sql, AspModelPO.class, catalog, parentId, us.getSiteId(), type);
      result = R.success(list);

      AspModelPO parent = dbEngine.queryEntity("select model_id, model_mc_id, model_parent_id, model_key, model_name, model_status, model_version, model_create_time, model_last_time from sw_bpm.asp_model" +
              " where model_id=?",AspModelPO.class, parentId);

      result.put("parent", parent);
    }

    return result;
  }

  public R move(@SwBody TreeMoveVO vo, UserSession us) {
    if (vo.getId() != null && vo.getOrders() != null) {
      List<Object[]> params = new ArrayList<>();

      int index = 0;
      for (Long order: vo.getOrders()) {
        params.add(new Object[]{vo.getParentId(), ++index, order});
      }

      int[] ret = dbEngine.batchUpdate("update sw_bpm.asp_model set model_parent_id=?, model_order=? where model_id=?", params);

      if (ret.length > 0) {
        return R.success();
      }
    }

    return R.error();
  }
}
