package cc.smtweb.system.bpm.web.design.modelcatalog;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.system.bpm.engine.entity.AspModelCatalogPO;
import org.apache.commons.lang3.StringUtils;

@SwService
public class AspModelCatalogCardService {
  @SwParam
  private DbEngine dbEngine;

  public R load(@SwParam("id") long id, UserSession us) {
    String sql = "select mc_id, mc_parent_id, mc_code, mc_site_id, mc_name, mc_create_time, mc_last_time from sw_bpm.asp_model_catalog" +
            " where mc_id=? and mc_site_id=?";
    AspModelCatalogPO po = dbEngine.queryEntity(sql, AspModelCatalogPO.class, id, us.getSiteId());
    return R.success(po);
  }

  public R save(@SwBody AspModelCatalogPO po, UserSession us) {
    if (StringUtils.isBlank(po.getMcName())) {
      return R.error("必须填写项目名称");
    }

    if (StringUtils.isBlank(po.getMcModule())) {
      return R.error("必须填写模块名");
    }

    Long oldId = dbEngine.queryLong("select mc_id from sw_bpm.asp_model_catalog where mc_module=? and mc_id<>?",
            po.getMcModule(), po.getMcId() == null ? 0 : po.getMcId());
    if (oldId != null) {
      return R.error("模块名有重复");
    }

    po.setMcLastTime(DateUtil.nowDateTimeLong());
    if (po.getMcId() == null || po.getMcId() == 0L) {
      po.setMcId(dbEngine.nextId());

      SqlBuilder.createInsert("sw_bpm.asp_model_catalog")
              .add("mc_id", po.getMcId())
              .add("mc_name", po.getMcName())
              .add("mc_module", po.getMcModule())
              .add("mc_site_id", us.getSiteId())
              .add("mc_create_time", po.getMcLastTime())
              .add("mc_last_time", po.getMcLastTime())
              .update(dbEngine);

      return R.success(po.getMcId());
    } else {
      SqlBuilder.createUpdate("sw_bpm.asp_model_catalog")
              .add("mc_name", po.getMcName())
              .add("mc_module", po.getMcModule())
              .add("mc_last_time", po.getMcLastTime())
              .addWhere("mc_id", po.getMcId())
              .addWhere("mc_site_id", us.getSiteId())
              .update(dbEngine);

      return R.success();
    }
  }

  public R delete(@SwBody AspModelCatalogPO modelPo, UserSession us) {
    Long id = dbEngine.queryLong("select model_id from sw_bpm.asp_model where model_mc_id=? limit 1", modelPo.getMcId());
    if (id != null) {
      return R.error("已有页面存在，不允许删除项目");
    }

    String sql = "delete from sw_bpm.asp_model_catalog where mc_id=? and mc_site_id=?";
    if (dbEngine.update(sql, modelPo.getMcId(), us.getSiteId()) > 0) {
      return R.success();
    }

    return R.error("删除失败");
  }
}
