package cc.smtweb.system.bpm.web.design.table;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelCatalog;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.AbstractTreeHandler;
import cc.smtweb.framework.core.mvc.service.TreeHelper;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.CommUtil;

import java.util.*;

/**
 * Created by Akmm at 2022/3/21 18:22
 */
public class ModelCatalogTreeHandler extends AbstractTreeHandler<Object> {
    //查询类型：0-目录；1-表定义;2-页面定义
    private final static int TYPE_CATALOG = 0;
    private final static int TYPE_TABLE = 1;
    private final static int TYPE_PAGE = 2;

    private long prj_id;//所属项目
    private int type;//查询类型：0-目录；1-表定义;2-页面定义

    private ModelCatalogTreeHelper mcTreeHelper = null;

    @Override
    public void init(SwMap params, UserSession us) {
        super.init(params, us);
        type = params.readInt("type");
        prj_id = params.readLong("prj_id");
        mcTreeHelper = (ModelCatalogTreeHelper) TreeHelper.getTreeHelper(ModelCatalog.ENTITY_NAME, ModelCatalogTreeHelper.class);
    }

    @Override
    protected List<Object> getChildren(Object bean) {
        if (bean instanceof ModelCatalog) {
            return super.getChildren(bean);
        }
        return null;
    }

    @Override
    protected List<Object> filterData() {
        List<Object> listRet = new ArrayList<>();

        EntityDao<ModelCatalog> dao = DbEngine.getInstance().findDao(ModelCatalog.ENTITY_NAME);
        String text = "%" + params.readString("text") + "%";
        List<ModelCatalog> list = dao.queryWhere(" mc_prj_id=? and (mc_name like ? or mc_code like ?) order by mc_name", prj_id, text, text);
        listRet.addAll(list);

        switch (type) {
            case TYPE_TABLE:
                EntityDao<ModelTable> tabledao = DbEngine.getInstance().findDao(ModelTable.class);
                List<ModelTable> l = tabledao.queryWhere(" tb_prj_id=? and (tb_name like ? or tb_title like ?) order by tb_name", prj_id, text, text);
                listRet.addAll(l);
                break;
            case TYPE_PAGE:
                ;
        }
        return listRet;
    }

    @Override
    protected List<Object> getChildren(long id) {
        List<Object> listRet = new ArrayList<>();
        List<ModelCatalog> list;
        list = mcTreeHelper.getChildren(id, prj_id, (o1, o2) -> CommUtil.chineseCompare(o1.getName(), o2.getName()));

        listRet.addAll(list);

        switch (type) {
            case TYPE_TABLE:
                addTableChildren(listRet, id);
                break;
            case TYPE_PAGE:
                addPageChildren(listRet, id);
        }
        return listRet;
    }

    //增加表
    private void addTableChildren(List<Object> listRet, long id) {
        Collection<ModelTable> set = ModelTableCache.getInstance().getTablesByMc(id, (o1, o2) -> CommUtil.chineseCompare(o1.getName(), o2.getName()));
        if (set == null || set.isEmpty()) return;

        listRet.addAll(set);
    }

    //增加页面定义
    private void addPageChildren(List<Object> listRet, long id) {

    }

    @Override
    protected long getId(Object bean) {
        if (bean instanceof DefaultEntity) {
            return ((DefaultEntity) bean).getEntityId();
        }
        if (bean instanceof ModelTable) {
            return ((ModelTable) bean).getId();
        }
        return 0;
    }

    @Override
    protected String getText(Object bean) {
        if (bean instanceof ModelCatalog) {
            return ((ModelCatalog) bean).getName();
        }
        if (bean instanceof ModelTable) {
            return ((ModelTable) bean).getTitle();
        }

        /*if (bean instanceof ModelTable) {
            return ((ModelTable)bean).getTitle();
        }*/
        return null;
    }

    @Override
    protected void buildNode(SwMap node, Object bean) {
        super.buildNode(node, bean);
        if (bean instanceof ModelCatalog) {
            node.put("type", TYPE_CATALOG);
        } else if (bean instanceof ModelTable) {
            node.put("type", TYPE_TABLE);
        }/* else if (bean instanceof ModelTable) {
            node.put("type", TYPE_TABLE);
        }*/
    }
}
