package cc.smtweb.system.bpm.web.task.service;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.system.bpm.engine.process.impl.AbstractTaskCardService;
import cc.smtweb.system.bpm.engine.process.impl.entity.LoadTaskAckVO;
import cc.smtweb.system.bpm.engine.process.runtime.ProcessInstance;
import cc.smtweb.system.bpm.engine.process.task.Task;
import cc.smtweb.system.bpm.engine.process.vo.FlowAction;
import cc.smtweb.system.bpm.engine.process.vo.FlowNode;
import cc.smtweb.system.bpm.engine.process.vo.FlowNodeField;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

// "工作流任务接口, 手机界面"
@SwService
public class BpmTaskCardService extends AbstractTaskCardService {
  @Override
  protected LoadTaskAckVO loadForm(Task task) {
    ProcessInstance processInstance = task.getProcessInstance();
    FlowNode flowNode = processInstance.getCurNode();

    LoadTaskAckVO data = new LoadTaskAckVO();

    loadFormStructAndData(task, data);

    // 动作按钮
    List<FlowAction> flowActions = flowNode.getActions();
    if (flowActions != null) {
      List<SwMap> uiActions = data.getAction();

      for (FlowAction flowAction: flowActions) {
        SwMap action = new SwMap(4);

        action.put("id", flowAction.getId());
        action.put("label", flowAction.getLabel());
        action.put("type", flowAction.getType());
        switch (flowAction.getType()) {
          case FlowAction.TYPE_COMMIT:
            action.put("btnType", "primary");
            break;
          case FlowAction.TYPE_REJECT:
          case FlowAction.TYPE_FINISH:
            action.put("btnType", "danger");
            break;
          case FlowAction.TYPE_TRANSMIT:
          case FlowAction.TYPE_RETURN:
            action.put("btnType", "warning");
            break;
          case FlowAction.TYPE_SAVE:
            action.put("btnType", "info");
            break;
          default:
            action.put("btnType", "default");
            break;
        }

        uiActions.add(action);
      }
    }

    return data;
  }

  private void loadFormStructAndData(Task task, LoadTaskAckVO data) {
    ProcessInstance processInstance = task.getProcessInstance();
    FlowNode flowNode = processInstance.getCurNode();

    List<FlowNodeField> fields = flowNode.getFields();

    if (fields != null) {
      // 读取数据
      SwMap body = loadFormData(task);

      // 生成界面结构和数据
      Map<String, Object> formData = data.getData();
      List<SwMap> uiFields = data.getForm();
      ModelTable modelTable = processInstance.getModelTable();
      for (FlowNodeField nodeField: fields) {
        ModelField modelField = modelTable.findField(nodeField.getId());
        int perm = nodeField.getPerm();

        if (modelField != null && (perm == FlowNodeField.PERM_READ || perm == FlowNodeField.PERM_WRITE)) {
          SwMap uiField = new SwMap(8);
          uiField.put("id", modelField.getName());

          if (perm == FlowNodeField.PERM_READ) {
            uiField.put("component", "fy-label");

            if (body != null) {
              // 从历史数据中读取
              Object value = body.get(modelField.getName());
              if (value != null) {
                formData.put(modelField.getName(), value);
              }
            }
          } else {
            switch (modelField.getDataType()) {
              case SwEnum.DataType.TYPE_BOOL:
                uiField.put("component", "fy-checkbox");
                break;
              case SwEnum.DataType.TYPE_DATETIME:
              case SwEnum.DataType.TYPE_DATE:
                uiField.put("component", "fy-datetime");
                break;
              default:
                uiField.put("component", "fy-text");
                break;
            }

            if (task.getLockRev() == 0) {
              if (StringUtils.isNotBlank(nodeField.getDefaultValue())) {
                formData.put(modelField.getName(), nodeField.getDefaultValue());
              }
            } else if (body != null) {
              // 从历史数据中读取
              Object value = body.get(modelField.getName());
              if (value != null) {
                formData.put(modelField.getName(), value);
              }
            }
          }

          SwMap uiFieldProps = new SwMap(16);
          uiField.put("props", uiFieldProps);

          uiFieldProps.put("name", modelField.getName());
          uiFieldProps.put("label", modelField.getTitle());
          uiFieldProps.put("placeholder", modelField.getTitle());
          if (nodeField.isNotNull()) {
            uiFieldProps.put("required", true);
          }

          uiFields.add(uiField);
        }
      }
    } // else {R.error("读取节点字段权限, id=" + flowNode.getId());}
  }

  protected SwMap loadFormData(Task task) {
    if (task.getId() != 0) {
      ProcessInstance processInstance = task.getProcessInstance();
      return this.processEngine.getFormDataStore().load(processInstance.getId(), processInstance.getRootId());
    }

    return null;
  }

}
