package cc.smtweb.system.bpm.web.ui.dao;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.sqlbuilder.SelectSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.core.ui.builder.SqlUtil;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmField;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldFilter;
import cc.smtweb.system.bpm.core.ui.entity.dataset.IBpmDatasetConst;
import cc.smtweb.system.bpm.engine.ui.entity.po.DynFormLogPO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

@Slf4j
public abstract class AbstractJsonDatasetPlugin {
  @SwParam
  protected DbEngine dbEngine;

  protected SelectSqlBuilder makeSqlBuilderForJson(BpmDataset dataset, SwMap body) {
    SelectSqlBuilder sqlBuilder = SqlBuilder.createSelect("sw_bpm.bpm_dyn_form_log");
    sqlBuilder.addWhere("dfl_table", dataset.fullName());

    List<Map<String, Object>> query = (List<Map<String, Object>>)body.get(SqlUtil.PN_FILTER);
    if (query != null) {
      addSqlWhereForJson(sqlBuilder, query, false);
    }

    List<Map<String, Object>> filters = (List<Map<String, Object>>)body.get(SqlUtil.PN_SEARCH);

    if (filters != null) {
      int filterType = body.readInt(SqlUtil.PN_SEARCH_TYPE, 0);
      addSqlWhereForJson(sqlBuilder, filters, filterType != 0);
    }

    return sqlBuilder;
  }

  protected SelectSqlBuilder makeSqlBuilderForJson(BpmDataset dataset, List<BpmFieldFilter> filters) {
    SelectSqlBuilder sqlBuilder = SqlBuilder.createSelect("sw_bpm.bpm_dyn_form_log");
    sqlBuilder.addWhere("dfl_table", dataset.fullName());

    if (filters != null) {
      for (BpmFieldFilter filter : filters) {
        String keyword = filter.getKeyword();
        String fieldName = filter.getField();

        if (StringUtils.isNotBlank(keyword) && StringUtils.isNotBlank(fieldName)) {
          SqlUtil.addSqlWhere(sqlBuilder, sqlJsonField(fieldName), filter.getOp(), keyword);
        }
      }
    }

    return sqlBuilder;
  }

  private void addSqlWhereForJson(SqlBuilder sqlBuilder, List<Map<String, Object>> filters, boolean whereOr) {
    if (!filters.isEmpty()) {
      if (whereOr) {
        // START OR
        sqlBuilder.addWhereOrBegin();
      }

      for (Map<String, Object> filter : filters) {
        Object keyword = filter.get(IBpmDatasetConst.PN_ITEM_KEYWORD);
        String field = (String) filter.get(IBpmDatasetConst.PN_ITEM_FIELD);

        if (keywordIsNotBlank(keyword) && StringUtils.isNotBlank(field)) {
          int op = MapUtils.getIntValue(filter, IBpmDatasetConst.PN_ITEM_OP, 0);
          SqlUtil.addSqlWhere(sqlBuilder, sqlJsonField(field), op, keyword);
        }
      }

      if (whereOr) {
        // END OR
        sqlBuilder.addWhereOrEnd();
      }
    }
  }

  private static boolean keywordIsNotBlank(Object keyword) {
    if (keyword instanceof String) {
      return StringUtils.isNotBlank((String)keyword);
    }

    return ObjectUtils.isNotEmpty(keyword);
  }

  SwMap mapFields(BpmDataset dataset, DynFormLogPO po) {
    if (StringUtils.isNotBlank(po.getDflContent())) {
      SwMap item = new SwMap();
      SwMap entity = JsonUtil.parse(po.getDflContent(), SwMap.class);

      for (Map.Entry<String, BpmField> entry : dataset.getFields().entrySet()) {
        BpmField bpmField = entry.getValue();
        item.put(bpmField.getId(), entity.get(bpmField.getFieldName()));
      }

      item.put("id", po.getDflId());

      return item;
    }

    return null;
  }

  static String sqlJsonField(String field) {
    return "JSON_EXTRACT(dfl_content, '$." + field + "')";
  }
}
