package cc.smtweb.system.bpm.web.ui.dao;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwBean;
import cc.smtweb.framework.core.db.sqlbuilder.InsertSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.UpdateSqlBuilder;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.builder.CardDataBuilder;
import cc.smtweb.system.bpm.core.ui.builder.SqlUtil;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmField;
import cc.smtweb.system.bpm.core.ui.entity.dataset.IBpmDatasetConst;
import cc.smtweb.system.bpm.engine.ui.entity.po.DynFormLogPO;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

@SwBean
public class JsonDatasetCardDao extends AbstractJsonDatasetPlugin {
  public SwMap load(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    SwMap body = ctx.getBody();
    long id = ctx.getParamId();

    if (id != 0) {
      DynFormLogPO po = dbEngine.queryEntity("select dfl_id, dfl_content from sw_bpm.bpm_dyn_form_log where dfl_id=?",
              DynFormLogPO.class, id);

      if (po != null){
        return mapFields(dataset, po);
      }
    } else {
      List<Map<String, Object>> filters = (List<Map<String, Object>>)body.get(SqlUtil.PN_FILTER);
      if (filters != null) {
        SwMap data = new SwMap();
        // addSqlWhereForJson(sqlBuilder, query, false);
        for (Map<String, Object> filter : filters) {
          String keyword = (String) filter.get(IBpmDatasetConst.PN_ITEM_KEYWORD);
          String field = (String) filter.get(IBpmDatasetConst.PN_ITEM_FIELD);

          if (StringUtils.isNotBlank(keyword) && StringUtils.isNotBlank(field)) {
            BpmField bpmField = dataset.findFieldByFieldName(field);
            if (bpmField != null) {
              data.put(bpmField.getId(), keyword);
            }
          }
        }

        return data;
      }
    }

    return null;
  }

  public Long save(BpmPageContext ctx) {
    CardDataBuilder builder = CardDataBuilder.create(dbEngine, ctx.getDataset(), ctx.getBody(), ctx.getSession());
    long dataId = builder.getMasterId();
    UserSession us = ctx.getSession();
    SwMap body = null;

    // TODO 处理1:1多表更新
    if (!builder.isInstert()) {
      // 读取旧数据进行部分更新
      String oldContent = dbEngine.queryString("select dfl_content from sw_bpm.bpm_dyn_form_log where dfl_id=?", dataId);

      if (StringUtils.isNotBlank(oldContent)) {
        body = JsonUtil.parse(oldContent, SwMap.class);
      }
    }

    String content = JsonUtil.encodeString(builder.update(body));

    if (builder.isInstert()) {
      InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.bpm_dyn_form_log");
      sqlBuilder.add("dfl_id", dataId)
              .add("dfl_table", builder.getTableName())
              .add("dfl_content", content)
              .add("dfl_create_uid", us.getUserId())
              .add("dfl_update_uid", us.getUserId());
      sqlBuilder.update(dbEngine);
    } else {
      UpdateSqlBuilder sqlBuilder = SqlBuilder.createUpdate("sw_bpm.bpm_dyn_form_log");
      sqlBuilder.add("dfl_content", content)
              .add("dfl_last_time", DateUtil.nowDateTimeLong())
              .add("dfl_update_uid", us.getUserId())
              .addWhere("dfl_id", dataId);
      sqlBuilder.update(dbEngine);
    }

    return dataId;
  }

  public int remove(BpmPageContext ctx) {
    String ids = ctx.getBody().readString(IParamConst.PARAMS_IDS);

    if (StringUtils.isNotBlank(ids)) {
      return dbEngine.update("delete from sw_bpm.bpm_dyn_form_log where dfl_id in(" + ids + ")");
    } else {
      long id = ctx.getParamId();
      if (id != 0) {
        return dbEngine.update("delete from sw_bpm.bpm_dyn_form_log where dfl_id=?", ctx.getParamId());
      }
    }

    return 0;
  }

  public BpmKeyValue loadLookup(BpmPageContext ctx, Object key) {
    return dbEngine.queryEntity("select dfl_id id, " + ctx.getFieldLookup().getLabelField()
            + " value from sw_bpm.bpm_dyn_form_log where dfl_id=?", BpmKeyValue.class, key);
  }
}
