package cc.smtweb.system.bpm.web.ui.dao;

import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwBean;
import cc.smtweb.framework.core.db.sqlbuilder.SelectSqlBuilder;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmMapData;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.builder.SqlUtil;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmField;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmFieldLookup;
import cc.smtweb.system.bpm.engine.ui.entity.po.DynFormLogPO;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

@SwBean
public class JsonDatasetListDao extends AbstractJsonDatasetPlugin {
  public List<SwMap> list(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    SwMap query = ctx.getBody();
    // 列表不能用 mobilePage.id 过滤，存储的是编辑界面的pageId
    // 处理查询参数
    SelectSqlBuilder sqlBuilder = makeSqlBuilderForJson(dataset, query);

    sqlBuilder.add("dfl_id");
    sqlBuilder.add("dfl_content");
    sqlBuilder.addOrderBy("dfl_id");

    // 处理翻页参数
    int limit = query.readInt("pageSize", 20);
    int start = query.readInt("pageIndex", 0) * limit;

    List<DynFormLogPO> list = sqlBuilder.pagedQuery(dbEngine, DynFormLogPO.class, start, limit);

    if (list != null) {
      return makeListResult(dataset, list, dataset.isUseSummary() || list.size() >= limit);
    }

    return null;
  }

  private List<SwMap> makeListResult(BpmDataset dataset, List<DynFormLogPO> list, boolean loadCount) {
    List<SwMap> data = new ArrayList<>();
    for (DynFormLogPO po : list) {
      SwMap item = mapFields(dataset, po);

      if (item == null) {
        item = new SwMap();
        item.put("id", po.getDflId());
      }

      data.add(item);
    }

    return data;
  }

  public BpmMapData listLookup(BpmPageContext ctx, Set<String> idSet) {
    // 外键数据
    List<BpmKeyValue> list =  dbEngine.query("select dfl_id optKey, " + ctx.getFieldLookup().getLabelField()
                    + " optValue from sw_bpm.bpm_dyn_form_log where dfl_id in (" + String.join(",", idSet) + ")",
            BpmKeyValue.class);

    BpmMapData result = new BpmMapData(list.size());
    for (BpmKeyValue item: list) {
      result.put(item.getOptKey(), item.getOptValue());
    }

    return result;
  }

  public List<SwMap> listAll(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    BpmFieldLookup fieldLookup = ctx.getFieldLookup();
    // 处理查询参数
    SelectSqlBuilder sqlBuilder = makeSqlBuilderForJson(dataset, fieldLookup.getFilters());

    sqlBuilder.add("dfl_id");
    sqlBuilder.add("dfl_content");
    sqlBuilder.addOrderBy("dfl_id");

    List<DynFormLogPO> list = sqlBuilder.query(dbEngine, DynFormLogPO.class);

    return makeListResult(dataset, list, false);
  }

  public SwMap count(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    SelectSqlBuilder sqlBuilder = makeSqlBuilderForJson(dataset, ctx.getBody());
    sqlBuilder.add("count(*) total");

    for (Map.Entry<String, BpmField> entry: dataset.getFields().entrySet()) {
      String field = entry.getKey();
      BpmField bpmField = entry.getValue();
      String rule = bpmField.getSummary();

      if (StringUtils.isNotBlank(rule)) {
        sqlBuilder.add(SqlUtil.makeSummaryField(rule, sqlJsonField(field)) + " " + field);
      }
    }

    return sqlBuilder.queryEntity(dbEngine, SwMap.class);
  }
}
