package cc.smtweb.system.bpm.web.ui.dynform;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.sqlbuilder.AbstractSelectSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.spring.cache.BpmPageCache;
import cc.smtweb.system.bpm.spring.dao.DatasetConfigDao;

import java.util.List;

@SwService
public class TaskListService {
  @SwParam
  private DatasetConfigDao datasetConfigDao;

  @SwParam
  protected BpmPageCache bpmPageCache;

  public R list(@SwParam("pageId") long pageId, @SwParam("start") int start, @SwParam("limit") int limit, UserSession us) {
    BpmPage mobilePage = bpmPageCache.get(pageId);

    if (mobilePage != null) {
      return list(mobilePage, us);
    }

    return R.error("未找到定义的界面, id=" + pageId);
  }

  @SwParam
  protected DbEngine dbEngine;

  protected R list(BpmPage page, UserSession us) {
    if (page.isDeploy()) {
      BpmDataset dataset = page.findMasterDataset();

      if (dataset != null) {
        return queryList(dataset.getDatabase(), dataset.getTable(), dataset.getFieldPrefix(), us.getUserId(), 0);
      }
    } else {
      return queryList("sw_bpm", "bpm_process_detail_log", "pdl", us.getUserId(), page.getId());
    }

    return R.error("界面未定义数据集");
  }

  private R queryList(String database, String table, String fieldPrefix, long userId, long pageId) {
    AbstractSelectSqlBuilder sb = SqlBuilder.createSelect(database, table)
      .add(fieldPrefix + "_id", null)
      .add(fieldPrefix + "_create_time", null)
      .add(fieldPrefix + "_last_time", null)
      .addWhere(fieldPrefix + "_update_uid", userId);

    if (pageId != 0) {
      sb.addWhere(fieldPrefix + "_process_root_id", pageId);
    }

    List<SwMap> list = sb.query(dbEngine, SwMap.class);

    return R.success(list).put("fieldPrefix", fieldPrefix);
  }
}
