package cc.smtweb.system.bpm.test.codegen.test;

import cc.smtweb.system.bpm.engine.ui.codegen.Sha256Helper;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.io.InputStream;

/**
 * 文件签名比较
 */
public class FileCompareTest {
  @Test
  public void testCompare() throws IOException {
    try (DataReader source = new DataReader("code/index.vue");
         DataReader target = new DataReader("code/index.vue.tmp")) {
      while (source.hasNext() || target.hasNext()) {
        byte sourceChar = source.next();
        byte targetChat = target.next();
        if (sourceChar != targetChat) {
          System.out.println(sourceChar + " <> " + targetChat);
          break;
        }
      }

      if (source.hasNext() || target.hasNext()) {
        Assertions.fail("source line: " + source.getLine() + ", target line: " + target.getLine());
      }
    }
  }
}

class DataReader implements AutoCloseable {
  private InputStream is;
  private int line = 1;
  private byte[] buffer = new byte[4096];
  private int readBytes;
  private int readIndex;

  public DataReader(String resource) throws IOException {
    ClassLoader loader = Thread.currentThread().getContextClassLoader();
    is = loader.getResourceAsStream(resource);
    readBytes = is.read(buffer);
  }

  @Override
  public void close() throws IOException {
    is.close();
  }

  public boolean hasNext() {
    return readIndex < readBytes;
  }

  public byte next() throws IOException {
    while (readIndex < readBytes) {
      byte ch = buffer[readIndex];

      if (++readIndex >= readBytes) {
        readBytes = is.read(buffer);
        readIndex = 0;
      }

      if (ch < 0) {
        return ch;
      }

      int type = Sha256Helper.SPACE_CHAR[ch];

      if (type == 0) {
        return ch;
      } else if (type == 3) {
        line++;
      }
    }

    return 0;
  }

  public int getLine() {
    return line;
  }
}
